import binascii

from .digest_algorithm import DigestAlgorithm
from .utils import _base64_decode


class DigestAlgorithmAndValue(object):

    def __init__(self, model):

        self.__algorithm = None
        algorithm = model.get('algorithm', None)
        if algorithm is not None:
            self.__algorithm = \
                DigestAlgorithm.get_instance_by_api_model(str(algorithm))

        self.__value = None
        value = model.get('value', None)
        if value is not None:
            self.__value = _base64_decode(value)

    @property
    def algorithm(self):
        return self.__algorithm

    @algorithm.setter
    def algorithm(self, value):
        self.__algorithm = value

    @property
    def value(self):
        return self.__value

    @value.setter
    def value(self, v):
        self.__value = v

    @property
    def hex_value(self):
        return binascii.hexlify(self.__value)

    @hex_value.setter
    def hex_value(self, value):
        self.__value = binascii.unhexlify(value)

    def to_model(self):
        return {
            'algorithm': self.__algorithm.api_model,
            'value': self.__value
        }


__all__ = ['DigestAlgorithmAndValue']



class PadesSize(object):

    def __init__(self, width=None, height=None):
        self.__width = width
        self.__height = height

    def to_model(self):
        return {
            'width': self.__width,
            'height': self.__height
        }

    @staticmethod
    def create_from_model(model):
        width = model.get('width', None)
        height = model.get('height', None)
        return PadesSize(width, height)


__all__ = ['PadesSize']



import re


class PkiBrazilCertificateFields(object):

    def __init__(self, model):
        self._certificate_type = model.get('certificateType', None)
        self._cpf = model.get('cpf', None)
        self._cnpj = model.get('cnpj', None)
        self._responsavel = model.get('responsavel', None)
        self._company_name = model.get('companyName', None)
        self._oab_uf = model.get('oabUF', None)
        self._oab_numero = model.get('oabNumero', None)
        self._rg_numero = model.get('rgNumero', None)
        self._rg_emissor = model.get('rgEmissor', None)
        self._rg_emissor_uf = model.get('rgEmissorUF', None)
        self._date_of_birth = model.get('dateOfBirth', None)

    @property
    def certificate_type(self):
        return self._certificate_type

    @certificate_type.setter
    def certificate_type(self, value):
        self._certificate_type = value

    @property
    def cpf(self):
        return self._cpf

    @cpf.setter
    def cpf(self, value):
        self._cpf = value

    @property
    def cpf_formatted(self):
        if self._cpf is None:
            return ''
        if not re.match('^\d{11}$', self._cpf):
            return self._cpf
        return "%s.%s.%s-%s" % (self._cpf[:3], self._cpf[3:6], self._cpf[6:9],
                                self._cpf[9:])

    @property
    def cnpj(self):
        return self._cnpj

    @cnpj.setter
    def cnpj(self, value):
        self._cnpj = value

    @property
    def cnpj_formatted(self):
        if self._cnpj is None:
            return ''
        if not re.match('^\d{14}', self._cnpj):
            return self._cnpj
        return "%s.%s.%s/%s-%s" % (self._cnpj[:2], self._cnpj[2:5],
                                   self._cnpj[5:8], self._cnpj[8:12],
                                   self._cnpj[12:])

    @property
    def responsavel(self):
        return self._responsavel

    @responsavel.setter
    def responsavel(self, value):
        self._responsavel = value

    @property
    def company_name(self):
        return self._company_name

    @company_name.setter
    def company_name(self, value):
        self._company_name = value

    @property
    def oab_uf(self):
        return self._oab_uf

    @oab_uf.setter
    def oab_uf(self, value):
        self._oab_uf = value

    @property
    def oab_numero(self):
        return self._oab_numero

    @oab_numero.setter
    def oab_numero(self, value):
        self._oab_numero = value

    @property
    def rg_numero(self):
        return self._rg_numero

    @rg_numero.setter
    def rg_numero(self, value):
        self._rg_numero = value

    @property
    def rg_emissor(self):
        return self._rg_emissor

    @rg_emissor.setter
    def rg_emissor(self, value):
        self._rg_emissor = value

    @property
    def rg_emissor_uf(self):
        return self._rg_emissor_uf

    @rg_emissor_uf.setter
    def rg_emissor_uf(self, value):
        self._rg_emissor_uf = value

    @property
    def date_of_birth(self):
        return self._date_of_birth

    @date_of_birth.setter
    def date_of_birth(self, value):
        self._date_of_birth = value


__all__ = ['PkiBrazilCertificateFields']



class XmlIdResolutionTable(object):

    def __init__(self, include_xml_id_attribute=True):
        self._include_xml_id_attribute = include_xml_id_attribute
        self._element_id_attributes = []
        self._global_id_attributes = []

    @property
    def include_xml_id_attribute(self):
        return self._include_xml_id_attribute

    @include_xml_id_attribute.setter
    def include_xml_id_attribute(self, value):
        self._include_xml_id_attribute = value

    @property
    def element_id_attributes(self):
        return self._element_id_attributes

    @element_id_attributes.setter
    def element_id_attributes(self, value):
        self._element_id_attributes = value

    @property
    def global_id_attributes(self):
        return self._global_id_attributes

    @global_id_attributes.setter
    def global_id_attributes(self, value):
        self._global_id_attributes = value

    def add_global_id_attribute(self,
                                id_attribute_local_name,
                                id_attribute_namespace=None):
        self._global_id_attributes.append({
            'localName': id_attribute_local_name,
            'namespace': id_attribute_namespace
        })

    def set_element_id_attribute(self,
                                 element_local_name,
                                 element_namespace,
                                 id_attribute_local_name,
                                 id_attribute_namespace=None):
        self._element_id_attributes.append({
            'element': {
                'localName': element_local_name,
                'namespace': element_namespace
            },
            'attribute': {
                'localName': id_attribute_local_name,
                'namespace': id_attribute_namespace
            }
        })

    def to_model(self):
        return {
            'includeXmlIdAttribute': self._include_xml_id_attribute,
            'elementIdAttributes': self._element_id_attributes,
            'globalIdAttributes': self._global_id_attributes
        }


__all__ = ['XmlIdResolutionTable']



from .xml_signature_starter import XmlSignatureStarter
from .signature_start_result import SignatureStartResult


class DetachedResourceXmlSignatureStarter(XmlSignatureStarter):

    def __init__(self, client):
        super(DetachedResourceXmlSignatureStarter, self).__init__(client)
        self.__resource_content = None
        self.__resource_uri = None

    @property
    def resource_content(self):
        return self.__resource_content

    @property
    def resource_uri(self):
        return self.__resource_uri

    # region set_to_sign_detached_resource

    def set_to_sign_detached_resource_from_path(self, path, uri=None):
        self.__resource_uri = uri
        with open(path, 'rb') as f:
            self.__resource_content = f.read()

    def set_to_sign_detached_resource_from_content_raw(self, content_raw,
                                                       uri=None):
        if not content_raw:
            raise Exception('Invalid resource content')

        self.__resource_uri = uri
        self.__resource_content = content_raw

    def set_to_sign_detached_resource(self, path, uri=None):
        self.set_to_sign_detached_resource_from_path(path, uri)

    def set_to_sign_detached_resource_content(self,
                                              content_raw,
                                              uri):
        self.set_to_sign_detached_resource_from_content_raw(content_raw, uri)

    # endregion

    def start(self):
        return self.__start_common()

    def start_with_web_pki(self):
        return self.__start_common(True)

    def start_with_webpki(self):
        return self.start_with_web_pki()

    def __start_common(self, is_with_web_pki=False):
        request = self.__get_start_common_request(is_with_web_pki)
        response = \
            self._client.post('Api/XmlSignatures/DetachedResourceXmlSignatures',
                              request)
        return SignatureStartResult(response)

    def __get_start_common_request(self, is_with_web_pki=False):
        self._verify_common_parameters(is_with_web_pki)
        if self.__resource_content is None or len(self.__resource_content) == 0:
            raise Exception('The detached resource to sign was not set')
        request = {}
        self._fill_request(request)
        request['detachedResourceToSignContent'] = self.__resource_content
        request['detachedResourceReferenceUri'] = self.__resource_uri
        return request


__all__ = ['DetachedResourceXmlSignatureStarter']



class XmlInsertionOptions(object):
    APPEND_CHILD = 'AppendChild'
    PREPEND_CHILD = 'PrependChild'
    APPEND_SIBLING = 'AppendSibling'
    PREPEND_SIBLING = 'PrependSibling'


__all__ = ['XmlInsertionOptions']



from .apis import Apis
from .file_reference import FileReference
from .file_result import FileResult
from .pades_measurement_units import PadesMeasurementUnits
from .rest_pki_client import _get_api_version


class PdfMarker(object):

    def __init__(self, client):
        self.__client = client
        self.__file = None
        self.__measurement_units = PadesMeasurementUnits.CENTIMETERS
        self.__page_optimization = None
        self.__abort_if_signed = None
        self.__marks = []
        self.__force_blob_result = None

    # region "client" accessors

    @property
    def client(self):
        return self.__get_client()

    def __get_client(self):
        return self.__client

    @client.setter
    def client(self, value):
        self.__set_client(value)

    def __set_client(self, value):
        if value is None:
            raise Exception('The provided "client" is not valid')
        self.__client = value

    # endregion

    # region "file" accessors

    @property
    def file(self):
        return self.__get_file()

    def __get_file(self):
        return self.__file.file_desc

    @file.setter
    def file(self, value):
        self.__set_file(value)

    def __set_file(self, value):
        if value is None:
            raise Exception('The provided "file" is not valid')
        self.__file = FileReference.from_file(value)

    # endregion

    # region "file_path" accessors

    @property
    def file_path(self):
        return self.__get_file_path()

    def __get_file_path(self):
        return self.__file.path

    @file_path.setter
    def file_path(self, value):
        self.__set_file_path(value)

    def __set_file_path(self, value):
        if value is None:
            raise Exception('The provided "file_path" is not valid')
        self.__file = FileReference.from_path(value)

    # endregion

    # region "file_content" accessors

    @property
    def file_content(self):
        return self.__get_file_content()

    def __get_file_content(self):
        return self.__file.content_raw

    @file_content.setter
    def file_content(self, value):
        self.__set_file_content(value)

    def __set_file_content(self, value):
        if value is None:
            raise Exception('The provided "file_content" is not valid')
        self.__file = FileReference.from_content_raw(value)

    # endregion

    # region "file_base64" accessors

    @property
    def file_base64(self):
        return self.__get_file_base64()

    def __get_file_base64(self):
        return self.__file.content_base64

    @file_base64.setter
    def file_base64(self, value):
        self.__set_file_base64(value)

    def __set_file_base64(self, value):
        if value is None:
            raise Exception('The provided "file_base64" is not valid')
        self.__file = FileReference.from_content_raw(value)

    # endregion

    # region "file_blob_token" accessors

    @property
    def file_blob_token(self):
        return self.__get_file_blob_token()

    def __get_file_blob_token(self):
        return self.__file.blob_token

    @file_blob_token.setter
    def file_blob_token(self, value):
        self.__set_file_blob_token(value)

    def __set_file_blob_token(self, value):
        if value is None:
            raise Exception('The provided "file_blob_token" is not valid')
        self.__file = FileReference.from_blob(value)

    # endregion

    # region "file_result" accessors

    @property
    def file_result(self):
        return self.__get_file_result()

    def __get_file_result(self):
        result = FileResult(self.__client, self.__file.content_base64)
        return result

    @file_result.setter
    def file_result(self, value):
        self.__set_file_result(value)

    def __set_file_result(self, value):
        if value is None:
            raise Exception('The provided "file_result" is not valid')
        self.__file = FileReference.from_result(value)

    # endregion

    # region "measurements_units" accessors

    @property
    def measurements_units(self):
        return self.__get_measurements_units()

    def __get_measurements_units(self):
        return self.__measurements_units

    @measurements_units.setter
    def measurements_units(self, value):
        self.__set_measurements_units(value)

    def __set_measurements_units(self, value):
        if value is None:
            raise Exception('The provided "measurements_units" is not valid')
        self.__measurements_units = value

    # endregion

    # region "page_optimization" accessors

    @property
    def page_optimization(self):
        return self.__get_page_optimization()

    def __get_page_optimization(self):
        return self.__page_optimization

    @page_optimization.setter
    def page_optimization(self, value):
        self.__set_page_optimization(value)

    def __set_page_optimization(self, value):
        if value is None:
            raise Exception('The provided "page_optimization" is not valid')
        self.__page_optimization = value

    # endregion

    # region "abort_if_signed" accessors

    @property
    def abort_if_signed(self):
        return self.__get_abort_if_signed()

    def __get_abort_if_signed(self):
        return self.__abort_if_signed

    @abort_if_signed.setter
    def abort_if_signed(self, value):
        self.__set_abort_if_signed(value)

    def __set_abort_if_signed(self, value):
        if value is None:
            raise Exception('The provided "abort_if_signed" is not valid')
        self.__abort_if_signed = value

    # endregion

    # region "marks" accessors

    @property
    def marks(self):
        return self.__get_marks()

    def __get_marks(self):
        return self.__marks

    @marks.setter
    def marks(self, value):
        self.__set_marks(value)

    def __set_marks(self, value):
        if value is None:
            raise Exception('The provided "marks" is not valid')
        self.__marks = value

    # endregion

    def append_mark(self, mark):
        if self.__marks is None:
            self.__marks = []
        self.__marks.append(mark)

    # region "force_blob_result" accessors

    @property
    def force_blob_result(self):
        return self.__get_force_blob_result()

    def __get_force_blob_result(self):
        return self.__force_blob_result

    @force_blob_result.setter
    def force_blob_result(self, value):
        self.__set_force_blob_result(value)

    def __set_force_blob_result(self, value):
        if value is None:
            raise Exception('The provided "force_blob_result" is not valid')
        self.__force_blob_result = value

    # endregion

    def apply(self):

        api_version = _get_api_version(self.__client, Apis.ADD_PDF_MARKS)
        if api_version < 1:
            raise Exception('The PdfMarker class can only be used with '
                            'REST PKI 1.13 or later. Please contact technical '
                            'support to update your REST PKI.')
        request = {
            'marks': [mark.to_model() for mark in self.__marks],
            'measurementUnits': self.__measurement_units,
            'forceBlobResult': self.__force_blob_result,
            'abortIfSigned': self.__abort_if_signed,
            'file': self.__file.upload_or_reference(self.__client),
            'pageOptimization': self.__page_optimization.to_model()
                                if self.__page_optimization is not None
                                else None
        }
        response = self.__client.post('Api/Pdf/AddMarks', request)
        return FileResult(self.__client, response.get('file', None))


__all__ = ['PdfMarker']



from .pki_brazil_certificate_fields import PkiBrazilCertificateFields
from .pki_italy_certificate_fields import PkiItalyCertificateFields
from .name import Name


class PKCertificate(object):

    def __init__(self, model):
        self._email_address = model.get('emailAddress', None)
        self._serial_number = model.get('serialNumber', None)
        self._validity_start = model.get('validityStart', None)
        self._validity_end = model.get('validityEnd', None)
        self._subject_name = None
        self._issuer_name = None
        self._pki_brazil = None
        self._pki_italy = None
        self._issuer = None

        subject_name = model.get('subjectName', None)
        if subject_name is not None:
            self._subject_name = Name(subject_name)

        issuer_name = model.get('issuerName', None)
        if issuer_name is not None:
            self._issuer_name = Name(issuer_name)

        pki_brazil = model.get('pkiBrazil', None)
        if pki_brazil is not None:
            self._pki_brazil = PkiBrazilCertificateFields(pki_brazil)

        pki_italy = model.get('pkiItaly', None)
        if pki_italy is not None:
            self._pki_italy = PkiItalyCertificateFields(pki_italy)

        issuer = model.get('issuer', None)
        if issuer is not None:
            self._issuer = PKCertificate(issuer)

    @property
    def email_address(self):
        return self._email_address

    @email_address.setter
    def email_address(self, value):
        self._email_address = value

    @property
    def serial_number(self):
        return self._serial_number

    @serial_number.setter
    def serial_number(self, value):
        self._serial_number = value

    @property
    def validity_start(self):
        return self._validity_start

    @validity_start.setter
    def validity_start(self, value):
        self._validity_start = value

    @property
    def validity_end(self):
        return self._validity_end

    @validity_end.setter
    def validity_end(self, value):
        self._validity_end = value

    @property
    def subject_name(self):
        return self._subject_name

    @subject_name.setter
    def subject_name(self, value):
        self._subject_name = value

    @property
    def issuer_name(self):
        return self._issuer_name

    @issuer_name.setter
    def issuer_name(self, value):
        self._issuer_name = value

    @property
    def pki_brazil(self):
        return self._pki_brazil

    @pki_brazil.setter
    def pki_brazil(self, value):
        self._pki_brazil = value

    @property
    def pki_italy(self):
        return self._pki_italy

    @pki_italy.setter
    def pki_italy(self, value):
        self._pki_italy = value

    @property
    def issuer(self):
        return self._issuer

    @issuer.setter
    def issuer(self, value):
        self._issuer = value


__all__ = ['PKCertificate']



import six


@six.python_2_unicode_compatible
class ValidationResults(object):

    def __init__(self, model):
        self._errors = None
        self._warnings = None
        self._passed_checks = None

        if model.get('errors', None) is not None:
            self._errors = self._convert_items(model['errors'])

        if model.get('warnings', None) is not None:
            self._warnings = self._convert_items(model['warnings'])

        if model.get('passedChecks', None) is not None:
            self._passed_checks = self._convert_items(model['passedChecks'])

    @property
    def errors(self):
        return self._errors

    @errors.setter
    def errors(self, value):
        self._errors = value

    @property
    def warning(self):
        return self._warnings

    @warning.setter
    def warning(self, value):
        self._warnings = value

    @property
    def passed_checks(self):
        return self._passed_checks

    @passed_checks.setter
    def passed_checks(self, value):
        self._passed_checks = value

    @property
    def is_valid(self):
        return len(self._errors) == 0

    @property
    def checks_performed(self):
        return len(self._errors) + \
               len(self._warnings) + \
               len(self._passed_checks)

    @property
    def has_errors(self):
        return not len(self._errors) == 0

    @property
    def has_warnings(self):
        return not len(self._warnings) == 0

    @staticmethod
    def _convert_items(items):
        converted = list()
        for item in items:
            converted.append(ValidationItem(item))

        return converted

    @staticmethod
    def _join_items(items, indentation_level):
        text = ''
        is_first = True
        tab = '\t' * indentation_level

        for item in items:
            if is_first:
                is_first = False
            else:
                text += '\n'

            text += '%s- ' % tab
            text += item.to_string(indentation_level)

        return text

    @property
    def summary(self):
        return self.get_summary(0)

    def get_summary(self, indentation_level=0):
        tab = '\t' * indentation_level
        text = '%sValidation results: ' % tab

        if self.checks_performed == 0:
            text += 'no checks performed'
        else:
            text += '%s checks performed' % self.checks_performed

            if self.has_errors:
                text += ', %s errors' % len(self._errors)
            if self.has_warnings:
                text += ', %s warnings' % len(self._warnings)
            if len(self._passed_checks) > 0:
                if not self.has_errors and not self.has_warnings:
                    text += ', all passed'
                else:
                    text += ', %s passed' % len(self._passed_checks)

        return text

    def __str__(self):
        return self.to_string(0)

    def to_string(self, indentation_level):
        tab = '\t' * indentation_level
        text = ''
        text += self.get_summary(indentation_level)

        if self.has_errors:
            text += '\n%sErrors:\n' % tab
            text += self._join_items(self._errors, indentation_level)

        if self.has_warnings:
            text += '\n%sWarnings:\n' % tab
            text += self._join_items(self._warnings, indentation_level)

        if self._passed_checks is not None:
            text += '\n%sPassed checks:\n' % tab
            text += self._join_items(self._passed_checks, indentation_level)

        return text


class ValidationItem:

    def __init__(self, model):
        self._item_type = model.get('type', None)
        self._message = model.get('message', None)
        self._detail = model.get('detail', None)
        self._inner_validation_results = None

        if model.get('innerValidationResults', None) is not None:
            self._inner_validation_results = ValidationResults(
                model['innerValidationResults'])

    @property
    def item_type(self):
        return self._item_type

    @item_type.setter
    def item_type(self, value):
        self._item_type = value

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, value):
        self._message = value

    @property
    def detail(self):
        return self._detail

    @detail.setter
    def detail(self, value):
        self._detail = value

    @property
    def inner_validation_results(self):
        return self._inner_validation_results

    @inner_validation_results.setter
    def inner_validation_results(self, value):
        self._inner_validation_results = value

    def __str__(self):
        return self.__unicode__()

    def __unicode__(self):
        return self.to_string(0)

    def to_string(self, indentation_level):
        text = ''
        text += self._message

        if self._detail is not None and len(self._detail) > 0:
            text += ' (%s)' % self._detail

        if self._inner_validation_results is not None:
            text += '\n'
            text += ''
            text += self._inner_validation_results\
                .to_string(indentation_level + 1)

        return text


__all__ = [
    'ValidationResults',
    'ValidationItem'
]



from .pdf_mark_image import PdfMarkImage
from .pdf_mark_element import PdfMarkElement
from .pdf_mark_element_type import PdfMarkElementType


class PdfMarkImageElement(PdfMarkElement):

    def __init__(self, relative_container=None, image=None):
        super(PdfMarkImageElement, self).__init__(PdfMarkElementType.IMAGE,
                                                  relative_container)
        self.__image = image

    @property
    def image(self):
        return self.__image

    @image.setter
    def image(self, value):
        self.__image = value

    # region FluentAPI

    def with_image(self, image):
        self.__image = image
        return self

    def with_image_content(self, image_content, mime_type):
        self.__image = PdfMarkImage(image_content, mime_type)
        return self

    # endregion

    def to_model(self):
        model = super(PdfMarkImageElement, self).to_model()
        model['image'] = self.__image.to_model() \
            if self.__image is not None else None
        return model


__all__ = ['PdfMarkImageElement']



from .rest_base_error import RestBaseError


class ValidationError(RestBaseError):

    def __init__(self, verb, url, validation_results):
        RestBaseError.__init__(self,
                               __name__,
                               str(validation_results),
                               verb,
                               url)
        self._validation_results = validation_results

    @property
    def validation_results(self):
        return self._validation_results

    @validation_results.setter
    def validation_results(self, value):
        self._validation_results = value


__all__ = ['ValidationError']



class RestBaseError(Exception):

    def __init__(self, name, message, verb, url):
        super(RestBaseError, self).__init__(message)
        self._name = name
        self._verb = verb
        self._url = url

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def verb(self):
        return self._verb

    @verb.setter
    def verb(self, value):
        self._verb = value

    @property
    def url(self):
        return self._url

    @url.setter
    def url(self, value):
        self._url = value


__all__ = ['RestBaseError']



from .pades_signer_info import PadesSignerInfo


class PadesSignature(object):

    def __init__(self, model):
        self.__signers = []
        signers = model.get('signers', None)
        if signers is not None:
            self.__signers = [PadesSignerInfo(s) for s in signers]

    @property
    def signers(self):
        return self.__signers

    @signers.setter
    def signers(self, value):
        self.__signers = value


__all__ = ['PadesSignature']



from .pdf_container_definition import PdfContainerDefinition
from .pdf_text_section import PdfTextSection
from .pdf_mark_qr_code_element import PdfMarkQRCodeElement
from .pdf_mark_image_element import PdfMarkImageElement
from .pdf_mark_text_element import PdfMarkTextElement
from .pdf_mark import PdfMark


class PdfHelper(object):

    @staticmethod
    def container():
        return PdfContainerDefinition.Initial()

    @staticmethod
    def mark():
        return PdfMark()

    @staticmethod
    def text_element():
        return PdfMarkTextElement()

    @staticmethod
    def image_element():
        return PdfMarkImageElement()

    @staticmethod
    def qr_code_element():
        return PdfMarkQRCodeElement()

    @staticmethod
    def text_section(text=None):
        return PdfTextSection(text)


__all__ = ['PdfHelper']



from abc import ABCMeta
from abc import abstractmethod


class SignatureStarter(object):
    __metaclass__ = ABCMeta

    def __init__(self, client):
        self._client = client
        self._signer_certificate = None
        self._signature_policy_id = None
        self._security_context_id = None
        self._callback_argument = None
        self._ignore_revocation_status_unknown = None

    @property
    def client(self):
        return self._client

    @client.setter
    def client(self, value):
        self._client = value

    @property
    def signer_certificate(self):
        return self._signer_certificate

    @signer_certificate.setter
    def signer_certificate(self, value):
        self._signer_certificate = value

    # region "security_context_id" accessors

    @property
    def security_context_id(self):
        return self.__get_security_context_id()

    def __get_security_context_id(self):
        return self._security_context_id

    @property
    def security_context(self):
        return self.__get_security_context_id()

    @security_context_id.setter
    def security_context_id(self, value):
        self.__set_security_context_id(value)

    def __set_security_context_id(self, value):
        if value is None:
            raise Exception('The provided "security_context_id" is not valid')
        self._security_context_id = value

    @security_context.setter
    def security_context(self, value):
        self.__set_security_context_id(value)

    # endregion

    # region "signature_policy_id" accessors

    @property
    def signature_policy_id(self):
        return self.__get_signature_policy_id()

    def __get_signature_policy_id(self):
        return self._signature_policy_id

    @property
    def signature_policy(self):
        return self.__get_signature_policy_id()

    @signature_policy_id.setter
    def signature_policy_id(self, value):
        self.__set_signature_policy_id(value)

    def __set_signature_policy_id(self, value):
        if value is None:
            raise Exception('The provided "signature_policy_id" is not valid')
        self._signature_policy_id = value

    @signature_policy.setter
    def signature_policy(self, value):
        self.__set_signature_policy_id(value)

    # endregion

    @abstractmethod
    def start(self):
        pass

    @abstractmethod
    def start_with_web_pki(self):
        pass


__all__ = ['SignatureStarter']



# pylint: disable=missing-docstring
import hashlib
import requests
import simplejson as json

from distutils.version import StrictVersion
from six import BytesIO

from .apis import Apis
from .authentication import Authentication
from .blob_reference import BlobReference
from .rest_error import RestError
from .rest_pki_error import RestPkiError
from .rest_unreachable_error import RestUnreachableError
from .utils import _base64_encode_string
from .validation import ValidationResults
from .validation_error import ValidationError
from .version import __version__


def _get_api_version(client, api):
    v = client.rest_pki_version
    if v is None:
        v = client.try_get_endpoint_version(client.endpoint_url)

    if api == Apis.START_CADES:
        if v > StrictVersion("1.11"):
            return 3
        elif v > StrictVersion("1.10"):
            return 2
        else:
            return 1
    elif api == Apis.COMPLETE_CADES:
        if v > StrictVersion("1.11"):
            return 2
        else:
            return 1
    elif api == Apis.START_PADES:
        if v > StrictVersion("1.11"):
            return 2
        else:
            return 1
    elif api == Apis.COMPLETE_PADES:
        if v > StrictVersion("1.11"):
            return 2
        else:
            return 1
    elif api == Apis.MULTIPART_UPLOAD:
        if v > StrictVersion("1.11"):
            return 1
        else:
            return 0
    elif api == Apis.ADD_PDF_MARKS:
        if v > StrictVersion("1.13"):
            return 1
        else:
            return 0
    else:
        raise Exception("Unsupported operation.")


class RestPkiClient(object):

    def __init__(self, endpoint_url, access_token):
        self.__endpoint_url = endpoint_url
        self.__auth_token = access_token
        self.__multipart_upload_double_check = None
        self.__multipart_upload_threshold = 5 * 1024 * 1024  # 5 MB
        self.__rest_pki_version = None
        self.__endpoint_versions = {}

    def upload_or_read(self, file_desc):
        api_version = _get_api_version(self, Apis.MULTIPART_UPLOAD)
        if api_version == 0:
            return RestPkiClient._read(file_desc)
        return self._upload(file_desc)

    @staticmethod
    def _read(file_desc):
        return file_desc.read()

    def _upload(self, file_desc):

        # Begin the upload

        begin_url = self.__endpoint_url + 'Api/MultipartUploads'
        begin_response = requests.post(begin_url,
                                       headers=self.get_request_headers())
        self._check_response('GET', begin_url, begin_response)

        blob_token = begin_response.json().get('blobToken')
        blob_uri = "Api/MultipartUploads/%s" % blob_token
        part_size = begin_response.json().get('partSize')

        # Read the file part by part

        part_e_tags = []
        part_number = 0

        # Return to the start of the stream.
        file_desc.seek(0, 0)
        while True:
            buffer = file_desc.read(part_size)
            if buffer is None or len(buffer) == 0:
                # Reached end-of-file.
                break
            headers = self.get_request_headers()
            part_hash = hashlib.md5(buffer)
            headers['Content-MD5'] = _base64_encode_string(part_hash.digest())
            headers['Content-Type'] = 'application/octet-stream'
            part_url = '%s/%s' % (blob_uri, part_number)
            response = requests.post(self.__endpoint_url + part_url,
                                     data=buffer,
                                     headers=headers)
            self._check_response('POST', part_url, response)

            e_tag = response.headers['ETag']
            part_e_tags.append(e_tag)
            part_number += 1

        # Finish upload

        end_request = {
            'partETags': part_e_tags,
            'completeMD5': None
        }
        if self.__multipart_upload_double_check:
            file_desc.seek(0, 0)
            md5 = hashlib.md5(file_desc.read())
            digest = md5.digest()
            end_request['completeMD5'] = _base64_encode_string(digest)
        finish_url = self.__endpoint_url + blob_uri
        finish_response = requests.post(finish_url,
                                        data=json.dumps(end_request),
                                        headers=self.get_request_headers())
        self._check_response('POST', finish_url, finish_response)

        return blob_token

    def upload_file(self, file_desc):
        blob_token = self._upload(file_desc)
        return BlobReference(blob_token)

    def upload_file_from_path(self, path):
        with open(path, 'rb') as file_desc:
            return self.upload_file(file_desc)

    def upload_file_from_raw(self, content_raw):
        stream = BytesIO()
        stream.write(content_raw)
        stream.seek(0, 0)
        return self.upload_file(stream)

    def try_get_endpoint_version(self, endpoint_uri):
        if endpoint_uri in self.__endpoint_versions:
            return self.__endpoint_versions.get(endpoint_uri)

        try:
            response = requests.get('%sApi/System/Info' % self.__endpoint_url,
                                    headers=self.get_request_headers())
            version = StrictVersion(response.json().get('productVersion'))
        except RestError as e:
            return None

        self.__endpoint_versions[endpoint_uri] = version
        return version

    def get_request_headers(self):
        headers = {
            'Authorization': 'Bearer %s' % self.__auth_token,
            'Accept': 'application/json',
            'Content-Type': 'application/json',
            'X-RestPki-Client': 'Python %s' % __version__
        }
        return headers

    def post(self, url, data=None):
        verb = 'POST'
        try:

            if data:
                response = requests.post('%s%s' % (self.__endpoint_url, url),
                                         data=json.dumps(data),
                                         headers=self.get_request_headers())
            else:
                response = requests.post('%s%s' % (self.__endpoint_url, url),
                                         headers=self.get_request_headers())
        except Exception as e:
            raise RestUnreachableError(verb, url)

        self._check_response(verb, url, response)
        return response.json()

    def get(self, url, params=None):
        verb = 'GET'
        try:
            response = requests.get('%s%s' % (self.__endpoint_url, url),
                                    params=params,
                                    headers=self.get_request_headers())
        except Exception as e:
            raise RestUnreachableError(verb, url)

        self._check_response(verb, url, response)
        return response.json()

    def get_raw(self, url):
        request_url = '%s%s' % (self.__endpoint_url, url)
        res = requests.get(request_url,
                           stream=True,
                           headers=self.get_request_headers())
        return res

    @staticmethod
    def _check_response(verb, url, response):
        status_code = response.status_code
        if status_code < 200 or status_code > 299:
            try:
                response_body = response.json()
                if status_code == 422 and response_body.get('code', None):
                    if response_body.get('code', None) == 'ValidationError':
                        vr = ValidationResults(
                            response_body.get('validationResults', None))
                        error = ValidationError(verb, url, vr)
                    else:
                        error = RestPkiError(verb,
                                             url,
                                             response_body.get('code', None),
                                             response_body.get('detail', None))
                else:
                    error = RestError(verb,
                                      url,
                                      status_code,
                                      response_body.get('message', None))
            except Exception as e:
                error = RestError(verb, url, status_code)

            raise error

    def get_authentication(self):
        return Authentication(self)

    # region "endpoint_url" accessors

    @property
    def endpoint_url(self):
        return self.__get_endpoint_url()

    def __get_endpoint_url(self):
        return self.__endpoint_url

    @endpoint_url.setter
    def endpoint_url(self, value):
        self.__set_endpoint_url(value)

    def __set_endpoint_url(self, value):
        if value is None:
            raise Exception('The provided "endpoint_url" is not valid')
        self.__endpoint_url = value

    # endregion

    # region "auth_token" accessors

    @property
    def auth_token(self):
        return self.__get_auth_token()

    def __get_auth_token(self):
        return self.__auth_token

    @auth_token.setter
    def auth_token(self, value):
        self.__set_auth_token(value)

    def __set_auth_token(self, value):
        if value is None:
            raise Exception('The provided "auth_token" is not valid')
        self.__auth_token = value

    # endregion

    # region "multipart_upload_double_check" accessors

    @property
    def multipart_upload_double_check(self):
        return self.__get_multipart_upload_double_check()

    def __get_multipart_upload_double_check(self):
        return self.__multipart_upload_double_check

    @multipart_upload_double_check.setter
    def multipart_upload_double_check(self, value):
        self.__set_multipart_upload_double_check(value)

    def __set_multipart_upload_double_check(self, value):
        if value is None:
            raise Exception(
                'The provided "multipart_upload_double_check" is '
                'not valid')
        self.__multipart_upload_double_check = value

    # endregion

    # region "multipart_upload_threshold" accessors

    @property
    def multipart_upload_threshold(self):
        return self.__get_multipart_upload_threshold()

    def __get_multipart_upload_threshold(self):
        return self.__multipart_upload_threshold

    @multipart_upload_threshold.setter
    def multipart_upload_threshold(self, value):
        self.__set_multipart_upload_threshold(value)

    def __set_multipart_upload_threshold(self, value):
        if value is None:
            raise Exception(
                'The provided "multipart_upload_threshold" is not '
                'valid')
        self.__multipart_upload_threshold = value

    # endregion

    # region "rest_pki_version" accessors

    @property
    def rest_pki_version(self):
        return self.__get_rest_pki_version()

    def __get_rest_pki_version(self):
        return self.__rest_pki_version

    @rest_pki_version.setter
    def rest_pki_version(self, value):
        self.__set_rest_pki_version(value)

    def __set_rest_pki_version(self, value):
        if value is None:
            raise Exception('The provided "rest_pki_version" is not valid')
        self.__rest_pki_version = value

    # endregion


__all__ = [
    '_get_api_version',
    'RestPkiClient'
]



from datetime import datetime

from .validation import ValidationResults
from .digest_algorithm_and_value import DigestAlgorithmAndValue
from .signature_algorithm_and_value import SignatureAlgorithmAndValue
from .signature_policy_identifier import SignaturePolicyIdentifier
from .pk_certificate import PKCertificate


class CadesSignature(object):

    def __init__(self, model):
        self.__encapsulated_content_type = \
            model.get('encapsulatedContentType', None)
        self.__has_encapsulated_content = \
            model.get('hasEncapsulatedContent', None)

        self.__signers = []
        signers = model.get('signers', None)
        if signers is not None:
            self.__signers = [CadesSignerInfo(s) for s in signers]

    @property
    def encapsulated_content_type(self):
        return self.__encapsulated_content_type

    @encapsulated_content_type.setter
    def encapsulated_content_type(self, value):
        self.__encapsulated_content_type = value

    @property
    def has_encapsulated_content(self):
        return self.__has_encapsulated_content

    @has_encapsulated_content.setter
    def has_encapsulated_content(self, value):
        self.__has_encapsulated_content = value

    @property
    def signers(self):
        return self.__signers

    @signers.setter
    def signers(self, value):
        self.__signers = value


class CadesTimestamp(CadesSignature):

    def __init__(self, model):
        super(CadesTimestamp, self).__init__(model)
        self.__gen_time = model.get('genTime', None)
        self.__serial_number = model.get('serialNumber', None)
        self.__message_imprint = model.get('messageImprint', None)

    @property
    def gen_time(self):
        return self.__gen_time

    @gen_time.setter
    def gen_time(self, value):
        self.__gen_time = value

    @property
    def serial_number(self):
        return self.__serial_number

    @serial_number.setter
    def serial_number(self, value):
        self.__serial_number = value

    @property
    def message_imprint(self):
        return self.__message_imprint

    @message_imprint.setter
    def message_imprint(self, value):
        self.__message_imprint = value


class CadesSignerInfo(object):

    def __init__(self, model):
        self.__certified_date_reference = \
            model.get('certifiedDateReference', None)

        self.__signing_time = None
        signing_time = model.get('signingTime', None)
        if signing_time is not None:
            # Parse date string from ISO 8601 pattern.
            # Partial solution:
            # - Return a datetime without timezone information.
            # Reason:
            # - Python doesn't have a good support for parsing string with
            # timezone.
            s_time = signing_time[:-6]
            self.__signing_time = datetime.strptime(s_time, '%Y-%m-%dT%H:%M:%S')

        self.__message_digest = None
        message_digest = model.get('messageDigest', None)
        if message_digest is not None:
            self.__message_digest = DigestAlgorithmAndValue(message_digest)

        self.__signature = None
        signature = model.get('signature', None)
        if signature is not None:
            self.__signature = SignatureAlgorithmAndValue(signature)

        self.__certificate = None
        certificate = model.get('certificate', None)
        if certificate is not None:
            self.__certificate = PKCertificate(certificate)

        self.__signature_policy = None
        signature_policy = model.get('signaturePolicy', None)
        if signature_policy is not None:
            self.__signature_policy = \
                SignaturePolicyIdentifier(signature_policy)

        self.__timestamps = []
        timestamps = model.get('timestamps', None)
        if timestamps is not None:
            for timestamp in timestamps:
                self.__timestamps.append(CadesTimestamp(timestamp))

        self.__validation_results = None
        validation_results = model.get('validationResults', None)
        if validation_results is not None:
            self.__validation_results = ValidationResults(validation_results)

    @property
    def signing_time(self):
        return self.__signing_time

    @signing_time.setter
    def signing_time(self, value):
        self.__signing_time = value

    @property
    def certified_date_reference(self):
        return self.__certified_date_reference

    @certified_date_reference.setter
    def certified_date_reference(self, value):
        self.__certified_date_reference = value

    @property
    def message_digest(self):
        return self.__message_digest

    @message_digest.setter
    def message_digest(self, value):
        self.__message_digest = value

    @property
    def signature(self):
        return self.__signature

    @signature.setter
    def signature(self, value):
        self.__signature = value

    @property
    def certificate(self):
        return self.__certificate

    @certificate.setter
    def certificate(self, value):
        self.__certificate = value

    @property
    def signature_policy(self):
        return self.__signature_policy

    @signature_policy.setter
    def signature_policy(self, value):
        self.__signature_policy = value

    @property
    def timestamps(self):
        return self.__timestamps

    @timestamps.setter
    def timestamps(self, value):
        self.__timestamps = value

    @property
    def validation_results(self):
        return self.__validation_results

    @validation_results.setter
    def validation_results(self, value):
        self.__validation_results = value


__all__ = [
    'CadesSignature',
    'CadesSignerInfo',
    'CadesTimestamp'
]



from .pades_signature import PadesSignature
from .signature_explorer import SignatureExplorer


class PadesSignatureExplorer(SignatureExplorer):
    PDF_MIME_TYPE = 'application/pdf'

    def open(self):
        if self._signature_file is None:
            raise Exception('The signature file to be opened was not set')

        request = {}
        self._fill_request(request)
        response = self._client.post('Api/PadesSignatures/Open', request)
        return PadesSignature(response)


__all__ = ['PadesSignatureExplorer']



from .apis import Apis
from .rest_pki_client import _get_api_version
from .file_reference import FileReference
from .signature_starter import SignatureStarter
from .signature_start_result import SignatureStartResult


class PadesSignatureStarter(SignatureStarter):

    def __init__(self, client):
        super(PadesSignatureStarter, self).__init__(client)
        self.__bypass_marks_if_signed = True
        self.__pdf_marks = []
        self.__pdf_to_sign = None
        self.__measurement_units = None
        self.__page_optimization = None
        self.__visual_representation = None

    # region "pdf_to_sign" accessors

    @property
    def pdf_to_sign(self):
        return self.__get_pdf_to_sign()

    def __get_pdf_to_sign(self):
        return self.__pdf_to_sign.file_desc

    @pdf_to_sign.setter
    def pdf_to_sign(self, value):
        self.__set_pdf_to_sign(value)

    def __set_pdf_to_sign(self, value):
        if value is None:
            raise Exception('The provided "pdf_to_sign" is not valid')
        self.__pdf_to_sign = FileReference.from_file(value)

    # endregion

    # region "pdf_to_sign_path" accessors

    @property
    def pdf_to_sign_path(self):
        return self.__get_pdf_to_sign_path()

    def __get_pdf_to_sign_path(self):
        return self.__pdf_to_sign.path

    @pdf_to_sign_path.setter
    def pdf_to_sign_path(self, value):
        self.__set_pdf_to_sign_path(value)

    def __set_pdf_to_sign_path(self, value):
        if value is None:
            raise Exception('The provided "pdf_to_sign_path" is not valid')
        self.__pdf_to_sign = FileReference.from_path(value)

    def set_pdf_to_sign_from_path(self, path):
        self.__set_pdf_to_sign_path(path)

    def set_pdf_to_sign_path(self, path):
        self.set_pdf_to_sign_from_path(path)

    def set_pdf_to_sign(self, path):
        self.set_pdf_to_sign_path(path)


    # endregion

    # region "pdf_to_sign_content" accessors

    @property
    def pdf_to_sign_content(self):
        return self.__get_pdf_to_sign_content()

    def __get_pdf_to_sign_content(self):
        return self.__pdf_to_sign.content_raw

    @pdf_to_sign_content.setter
    def pdf_to_sign_content(self, value):
        self.__set_pdf_to_sign_content(value)

    def __set_pdf_to_sign_content(self, value):
        if value is None:
            raise Exception('The provided "pdf_to_sign_content" is not valid')
        self.__pdf_to_sign = FileReference.from_content_raw(value)

    def set_pdf_to_sign_from_content_raw(self, content_raw):
        self.__set_pdf_to_sign_content(content_raw)

    def set_pdf_to_sign_content(self, content_raw):
        self.set_pdf_to_sign_from_content_raw(content_raw)

    # endregion

    # region "pdf_to_sign_base64" accessors

    @property
    def pdf_to_sign_base64(self):
        return self.__get_pdf_to_sign_base64()

    def __get_pdf_to_sign_base64(self):
        return self.__pdf_to_sign.content_base64

    @pdf_to_sign_base64.setter
    def pdf_to_sign_base64(self, value):
        self.__set_pdf_to_sign_base64(value)

    def __set_pdf_to_sign_base64(self, value):
        if value is None:
            raise Exception('The provided "pdf_to_sign_base64" is not valid')
        self.__pdf_to_sign = FileReference.from_content_base64(value)

    def set_pdf_to_sign_from_content_base64(self, content_base64):
        self.__set_pdf_to_sign_base64(content_base64)

    # endregion

    @property
    def bypass_marks_if_signed(self):
        return self.__bypass_marks_if_signed

    @bypass_marks_if_signed.setter
    def bypass_marks_if_signed(self, value):
        self.__bypass_marks_if_signed = value

    @property
    def pdf_marks(self):
        return self.__pdf_marks

    @pdf_marks.setter
    def pdf_marks(self, value):
        self.__pdf_marks = value

    def add_mark(self, value):
        if self.__pdf_marks is None:
            self.__pdf_marks = []
        self.__pdf_marks.append(value)

    @property
    def measurement_units(self):
        return self.__measurement_units

    @measurement_units.setter
    def measurement_units(self, value):
        self.__measurement_units = value

    @property
    def page_optimization(self):
        return self.__page_optimization

    @page_optimization.setter
    def page_optimization(self, value):
        self.__page_optimization = value

    @property
    def visual_representation(self):
        return self.__visual_representation

    @visual_representation.setter
    def visual_representation(self, value):
        self.__visual_representation = value

    def start(self):
        if self._signer_certificate is None:
            raise Exception('The certificate was not set')
        response = self.__start_common()
        return SignatureStartResult(response)

    def start_with_web_pki(self):
        response = self.__start_common()
        return SignatureStartResult(response)

    def start_with_webpki(self):
        return self.start_with_web_pki()

    def __start_common(self):

        if not self.__pdf_to_sign:
            raise Exception('The PDF to be signed was not set')

        if not self._signature_policy_id:
            raise Exception('The signature policy was not set')

        api_version = _get_api_version(self._client, Apis.START_PADES)
        if api_version == 1:
            return self.__start_common_v1()
        return self.__start_common_v2()

    def __start_common_v2(self):
        request = self.__get_start_common_request()
        request['pdfToSign'] = \
            self.__pdf_to_sign.upload_or_reference(self._client)
        return self._client.post('Api/v2/PadesSignatures', request)

    def __start_common_v1(self):
        request = self.__get_start_common_request()
        request['pdfToSign'] = self.__pdf_to_sign.content_base64
        return self._client.post('Api/PadesSignatures', request)

    def __get_start_common_request(self):
        return {
            'certificate': self._signer_certificate,
            'signaturePolicyId': self._signature_policy_id,
            'securityContextId': self._security_context_id,
            'ignoreRevocationStatusUnknown':
                self._ignore_revocation_status_unknown,
            'callbackArgument': self._callback_argument,
            'pdfMarks': [m.to_model() for m in self.__pdf_marks],
            'bypassMarksIfSigned': self.__bypass_marks_if_signed,
            'measurementUnits': self.__measurement_units,
            'pageOptimization': self.__page_optimization.to_model()
                                if self.__page_optimization is not None
                                else None,
            'visualRepresentation': self.__visual_representation
        }


__all__ = ['PadesSignatureStarter']



from .color import Color
from .pdf_text_style import PdfTextStyle


class PdfTextSection(object):

    def __init__(self, text=None, color=None, font_size=None):
        self.__style = PdfTextStyle.NORMAL
        self.__text = text
        self.__font_size = font_size
        self.__color = color if color is not None else Color.BLACK

    @property
    def text(self):
        return self.__text

    @text.setter
    def text(self, value):
        self.__text = value

    @property
    def color(self):
        return self.__color

    @color.setter
    def color(self, value):
        self.__color = value

    @property
    def font_size(self):
        return self.__font_size

    @font_size.setter
    def font_size(self, value):
        self.__font_size = value

    @property
    def style(self):
        return self.__style

    @style.setter
    def style(self, value):
        self.__style = value

    # region FluentAPI

    def with_font_size(self, font_size):
        self.__font_size = font_size
        return self

    def with_text(self, text):
        self.__text = text
        return self

    def bold(self):
        self.__style = PdfTextStyle.BOLD
        return self

    def italic(self):
        self.__style = PdfTextStyle.ITALIC
        return self

    def with_color(self, color):
        self.__color = color
        return self

    # endregion

    def to_model(self):
        return {
            'style': self.__style,
            'text': self.__text,
            'color': self.__color.to_model() if
            self.__color is not None else None,
            'fontSize': self.__font_size
        }


__all__ = ['PdfTextSection']



from .apis import Apis
from .digest_algorithm import DigestAlgorithm
from .file_reference import FileReference
from .rest_pki_client import _get_api_version
from .signature_starter import SignatureStarter
from .signature_start_result import SignatureStartResult


class CadesSignatureStarter(SignatureStarter):

    def __init__(self, client):
        super(CadesSignatureStarter, self).__init__(client)
        self.__file_to_sign = None
        self.__cms_to_cosign = None
        self.__encapsulate_content = True
        self.__digest_algorithms_for_detached_signature = [
            DigestAlgorithm.SHA1,
            DigestAlgorithm.SHA256
        ]

    # region "file_to_sign" accessors

    @property
    def file_to_sign(self):
        return self.__get_file_to_sign()

    def __get_file_to_sign(self):
        return self.__file_to_sign.file_desc

    @file_to_sign.setter
    def file_to_sign(self, value):
        self.__set_file_to_sign(value)

    def __set_file_to_sign(self, value):
        if value is None:
            raise Exception('The provided "file_to_sign" is not valid')
        self.__file_to_sign = FileReference.from_file(value)

    # endregion

    # region "file_to_sign_path" accessors

    @property
    def file_to_sign_path(self):
        return self.__get_file_to_sign_path()

    def __get_file_to_sign_path(self):
        return self.__file_to_sign.path

    @file_to_sign_path.setter
    def file_to_sign_path(self, value):
        self.__set_file_to_sign_path(value)

    def __set_file_to_sign_path(self, value):
        if value is None:
            raise Exception('The provided "file_to_sign_path" is not valid')
        self.__file_to_sign = FileReference.from_path(value)

    def set_file_to_sign_from_path(self, path):
        self.__set_file_to_sign_path(path)

    def set_file_to_sign_path(self, path):
        self.set_file_to_sign_from_path(path)

    def set_file_to_sign(self, path):
        self.set_file_to_sign_path(path)

    # endregion

    # region "file_to_sign_base64" accessors

    @property
    def file_to_sign_base64(self):
        return self.__get_file_to_sign_base64()

    def __get_file_to_sign_base64(self):
        return self.__file_to_sign.content_base64

    @file_to_sign_base64.setter
    def file_to_sign_base64(self, value):
        self.__set_file_to_sign_base64(value)

    def __set_file_to_sign_base64(self, value):
        if value is None:
            raise Exception('The provided "file_to_sign_base64" is not valid')
        self.__file_to_sign_base64 = FileReference.from_content_base64(value)

    def set_file_to_sign_from_content_base64(self, content_base64):
        self.__set_file_to_sign_base64(content_base64)

    # endregion

    # region "file_to_sign_content" accessors

    @property
    def file_to_sign_content(self):
        return self.__get_file_to_sign_content()

    def __get_file_to_sign_content(self):
        return self.__file_to_sign_content

    @file_to_sign_content.setter
    def file_to_sign_content(self, value):
        self.__set_file_to_sign_content(value)

    def __set_file_to_sign_content(self, value):
        if value is None:
            raise Exception('The provided "file_to_sign_content" is not valid')
        self.__file_to_sign_content = value

    def set_file_to_sign_from_content_raw(self, content_raw):
        self.__set_file_to_sign_content(content_raw)

    def set_file_to_sign_content(self, content_raw):
        self.set_file_to_sign_from_content_raw(content_raw)

    # endregion
    
    # region "cms_to_cosign" accessors 
    
    @property
    def cms_to_cosign(self):
        return self.__get_cms_to_cosign()
        
    def __get_cms_to_cosign(self):
        return self.__cms_to_cosign.file_desc
    
    @cms_to_cosign.setter
    def cms_to_cosign(self, value):
        self.__set_cms_to_cosign(value)
        
    def __set_cms_to_cosign(self, value):
        if value is None:
            raise Exception('The provided "cms_to_cosign" is not valid')
        self.__cms_to_cosign = FileReference.from_file(value)
    
    # endregion
    
    # region "cms_to_cosign_path" accessors

    @property
    def cms_to_cosign_path(self):
        return self.__get_cms_to_cosign_path()

    def __get_cms_to_cosign_path(self):
        return self.__cms_to_cosign.path

    @cms_to_cosign_path.setter
    def cms_to_cosign_path(self, value):
        self.__set_cms_to_cosign_path(value)

    def __set_cms_to_cosign_path(self, value):
        if value is None:
            raise Exception('The provided "cms_to_cosign_path" is not valid')
        self.__cms_to_cosign = FileReference.from_path(value)

    def set_cms_to_cosign_from_path(self, path):
        self.__set_cms_to_cosign_path(path)

    def set_cms_to_cosign_path(self, path):
        self.set_cms_to_cosign_from_path(path)

    def set_cms_to_cosign(self, path):
        self.set_cms_to_cosign_path(path)

    # endregion

    # region "cms_to_cosign_content" accessors

    @property
    def cms_to_cosign_content(self):
        return self.__get_cms_to_cosign_content()

    def __get_cms_to_cosign_content(self):
        return self.__cms_to_cosign.content_raw

    @cms_to_cosign_content.setter
    def cms_to_cosign_content(self, value):
        self.__set_cms_to_cosign_content(value)

    def __set_cms_to_cosign_content(self, value):
        if value is None:
            raise Exception('The provided "cms_to_cosign_content" is not valid')
        self.__cms_to_cosign = FileReference.from_content_raw(value)

    def set_cms_to_cosign_from_content_raw(self, content_raw):
        self.__set_cms_to_cosign_content(content_raw)

    def set_cms_to_cosign_content(self, content_raw):
        self.set_cms_to_cosign_from_content_raw(content_raw)

    # endregion

    # region "cms_to_cosign_base64" accessors

    @property
    def cms_to_cosign_base64(self):
        return self.__get_cms_to_cosign_base64()

    def __get_cms_to_cosign_base64(self):
        return self.__cms_to_cosign.content_base64

    @cms_to_cosign_base64.setter
    def cms_to_cosign_base64(self, value):
        self.__set_cms_to_cosign_base64(value)

    def __set_cms_to_cosign_base64(self, value):
        if value is None:
            raise Exception('The provided "cms_to_cosign_base64" is not valid')
        self.__cms_to_cosign = FileReference.from_content_base64(value)

    def set_cms_to_cosign_from_content_base64(self, content_base64):
        self.__set_cms_to_cosign_base64(content_base64)

    # endregion

    # region "cms_to_cosign_result" accessors

    @property
    def cms_to_cosign_result(self):
        return self.__get_cms_to_cosign_result()

    def __get_cms_to_cosign_result(self):
        return self.__cms_to_cosign.result

    @cms_to_cosign_result.setter
    def cms_to_cosign_result(self, value):
        self.__set_cms_to_cosign_result(value)

    def __set_cms_to_cosign_result(self, value):
        if value is None:
            raise Exception('The provided "cms_to_cosign_result" is not valid')
        self.__cms_to_cosign = FileReference.from_result(value)

    # endregion

    @property
    def encapsulate_content(self):
        return self.__encapsulate_content

    @encapsulate_content.setter
    def encapsulate_content(self, value):
        self.__encapsulate_content = value

    def start(self):
        if self._signer_certificate is None:
            raise Exception('The certificate was not set')
        response = self.__start_common()
        return SignatureStartResult(response)

    def start_with_web_pki(self):
        response = self.__start_common()
        return SignatureStartResult(response)

    def start_with_webpki(self):
        return self.start_with_web_pki()

    def __start_common(self):

        if self.__file_to_sign is None and self.__cms_to_cosign is None:
            raise Exception('The content to sign was not set and no CMS to be '
                            'co-signed was given')

        if self._signature_policy_id is None:
            raise Exception('The signature policy was not set')

        api_version = _get_api_version(self._client, Apis.START_CADES)
        if api_version == 1:
            return self.__start_common_v1()
        elif api_version == 2:
            return self.__start_common_v2()
        return self.__start_common_v3()

    def __start_common_v3(self):
        request = self.__get_start_common_request()

        if self.__file_to_sign is not None:
            if not self.__encapsulate_content:
                request['dataHashes'] = self.__file_to_sign.compute_data_hashes(
                    self.__digest_algorithms_for_detached_signature)
            else:
                request['fileToSign'] = \
                    self.__file_to_sign.upload_or_reference(self._client)

        if self.__cms_to_cosign is not None:
            request['cmsToCoSign'] = \
                self.__cms_to_cosign.upload_or_reference(self._client)

        return self._client.post('Api/v3/CadesSignatures', request)

    def __start_common_v2(self):
        request = self.__get_start_common_request()

        if self.__file_to_sign is not None:
            if not self.__encapsulate_content:
                request['dataHashes'] = self.__file_to_sign.compute_data_hashes(
                    self.__digest_algorithms_for_detached_signature)
            else:
                request['contentToSign'] = self.__file_to_sign.content_base64

        if self.__cms_to_cosign is not None:
            request['cmsToCoSign'] = self.__file_to_sign.content_base64

        return self._client.post('Api/v2/CadesSignatures', request)

    def __start_common_v1(self):
        request = self.__get_start_common_request()

        if self.__file_to_sign is not None:
            request['contentToSign'] = self.__file_to_sign.content_base64

        if self.__cms_to_cosign is not None:
            request['cmsToCoSign'] = self.__file_to_sign.content_base64

        return self._client.post('Api/CadesSignatures', request)

    def __get_start_common_request(self):
        return {
            'certificate': self._signer_certificate,
            'signaturePolicyId': self._signature_policy_id,
            'securityContextId': self._security_context_id,
            'ignoreRevocationStatusUnknown':
                self._ignore_revocation_status_unknown,
            'callbackArgument': self._callback_argument,
            'encapsulateContent': self.__encapsulate_content
        }


__all__ = ['CadesSignatureStarter']



class PkiItalyCertificateFields(object):

    def __init__(self, model):
        self._certificate_type = model.get('certificateType', None)
        self._codice_fiscale = model.get('codiceFiscale', None)
        self._id_carta = model.get('idCarta', None)

    @property
    def certificate_type(self):
        return self._certificate_type

    @certificate_type.setter
    def certificate_type(self, value):
        self._certificate_type = value

    @property
    def codice_fiscale(self):
        return self._codice_fiscale

    @codice_fiscale.setter
    def codice_fiscale(self, value):
        self._codice_fiscale = value

    @property
    def id_carta(self):
        return self._id_carta

    @id_carta.setter
    def id_carta(self, value):
        self._id_carta = value


__all__ = ['PkiItalyCertificateFields']



from .standard_signature_policies import StandardSignaturePolicies


class SignaturePolicyCatalog(object):

    def __init__(self, policies=None):
        self.__policies = []
        if policies is not None:
            self.__policies.append(policies)

    @staticmethod
    def get_pki_brazil_cades():
        return SignaturePolicyCatalog([
            StandardSignaturePolicies.PKI_BRAZIL_CADES_ADR_BASICA,
            StandardSignaturePolicies.PKI_BRAZIL_CADES_ADR_TEMPO,
            StandardSignaturePolicies.PKI_BRAZIL_CADES_ADR_COMPLETA
        ])

    @staticmethod
    def get_pki_brazil_cades_with_signer_certificate_protection():
        return SignaturePolicyCatalog([
            StandardSignaturePolicies.PKI_BRAZIL_CADES_ADR_TEMPO,
            StandardSignaturePolicies.PKI_BRAZIL_CADES_ADR_COMPLETA
        ])

    @staticmethod
    def get_pki_brazil_cades_with_ca_certificate_protection():
        return SignaturePolicyCatalog([
            StandardSignaturePolicies.PKI_BRAZIL_CADES_ADR_COMPLETA
        ])

    @staticmethod
    def get_pki_brazil_pades():
        return SignaturePolicyCatalog([
            StandardSignaturePolicies.PKI_BRAZIL_PADES_ADR_BASICA,
            StandardSignaturePolicies.PKI_BRAZIL_PADES_ADR_TEMPO
        ])

    @staticmethod
    def get_pki_brazil_pades_with_signer_certificate_protection():
        return SignaturePolicyCatalog([
            StandardSignaturePolicies.PKI_BRAZIL_PADES_ADR_TEMPO
        ])

    @staticmethod
    def get_pki_brazil_xades():
        return SignaturePolicyCatalog([
            StandardSignaturePolicies.PKI_BRAZIL_XADES_ADR_BASICA,
            StandardSignaturePolicies.PKI_BRAZIL_XADES_ADR_TEMPO
        ])


__all__ = ['SignaturePolicyCatalog']



from abc import ABCMeta

from .file_result import FileResult
from .file_reference import FileReference


class SignatureExplorer(object):
    __metaclass__ = ABCMeta

    def __init__(self, client):
        self._client = client
        self._signature_file = None

        self.__validate = None
        self.__security_context_id = None
        self.__default_signature_policy_id = None
        self.__acceptable_explicit_policies = None
        self.__ignore_revocation_status_unknown = False

    # region "client" accessors

    @property
    def client(self):
        return self.__get_client()

    def __get_client(self):
        return self._client

    @client.setter
    def client(self, value):
        self.__set_client(value)

    def __set_client(self, value):
        if value is None:
            raise Exception('The provided "client" is not valid')
        self._client = value

    # endregion

    # region "signature_file" accessors

    @property
    def signature_file(self):
        return self.__get_signature_file()

    def __get_signature_file(self):
        return self._signature_file.file_desc

    @signature_file.setter
    def signature_file(self, value):
        self.__set_signature_file(value)

    def __set_signature_file(self, value):
        if value is None:
            raise Exception('The provided "signature_file" is not valid')
        self._signature_file = FileReference.from_file(value)

    # endregion

    # region "signature_file_path" accessors

    @property
    def signature_file_path(self):
        return self.__get_signature_file_path()

    def __get_signature_file_path(self):
        return self._signature_file.path

    @signature_file_path.setter
    def signature_file_path(self, value):
        self.__set_signature_file_path(value)

    def __set_signature_file_path(self, value):
        if value is None:
            raise Exception('The provided "signature_file_path" is not valid')
        self._signature_file = FileReference.from_path(value)

    # endregion

    # region "signature_file_raw" accessors

    @property
    def signature_file_raw(self):
        return self.__get_signature_file_raw()

    def __get_signature_file_raw(self):
        return self._signature_file.content_raw

    @signature_file_raw.setter
    def signature_file_raw(self, value):
        self.__set_signature_file_raw(value)

    def __set_signature_file_raw(self, value):
        if value is None:
            raise Exception('The provided "signature_file_raw" is not valid')
        self._signature_file = FileReference.from_content_raw(value)

    # endregion

    # region "signature_file_base64" accessors

    @property
    def signature_file_base64(self):
        return self.__get_signature_file_base64()

    def __get_signature_file_base64(self):
        return self._signature_file.content_base64

    @signature_file_base64.setter
    def signature_file_base64(self, value):
        self.__set_signature_file_base64(value)

    def __set_signature_file_base64(self, value):
        if value is None:
            raise Exception('The provided "signature_file_base64" is not valid')
        self._signature_file = FileReference.from_content_base64(value)

    # endregion

    # region "signature_file_blob_token" accessors

    @property
    def signature_file_blob_token(self):
        return self.__get_signature_file_blob_token()

    def __get_signature_file_blob_token(self):
        return self._signature_file.blob_token

    @signature_file_blob_token.setter
    def signature_file_blob_token(self, value):
        self.__set_signature_file_blob_token(value)

    def __set_signature_file_blob_token(self, value):
        if value is None:
            raise Exception('The provided "signature_file_blob_token" is not '
                            'valid')
        self._signature_file = FileReference.from_blob(value)

    # endregion

    # region "signature_file_result" accessors

    @property
    def signature_file_result(self):
        return self.__get_signature_file_result()

    def __get_signature_file_result(self):
        result = FileResult(self._client, self._signature_file.content_base64)
        return result

    @signature_file_result.setter
    def signature_file_result(self, value):
        self.__set_signature_file_result(value)

    def __set_signature_file_result(self, value):
        if value is None:
            raise Exception('The provided "signature_file_result" is not valid')
        self._signature_file = FileReference.from_result(value)

    # endregion

    # region "validate" accessors

    @property
    def validate(self):
        return self.__get_validate()

    def __get_validate(self):
        return self.__validate

    @validate.setter
    def validate(self, value):
        self.__set_validate(value)

    def __set_validate(self, value):
        if value is None:
            raise Exception('The provided "validate" is not valid')
        self.__validate = value

    # endregion

    # region "security_context_id" accessors

    @property
    def security_context_id(self):
        return self.__get_security_context_id()

    def __get_security_context_id(self):
        return self.__security_context_id

    @security_context_id.setter
    def security_context_id(self, value):
        self.__set_security_context_id(value)

    def __set_security_context_id(self, value):
        if value is None:
            raise Exception('The provided "security_context_id" is not valid')
        self.__security_context_id = value

    # endregion

    # region "default_signature_policy_id" accessors

    @property
    def default_signature_policy_id(self):
        return self.__get_default_signature_policy_id()

    def __get_default_signature_policy_id(self):
        return self.__default_signature_policy_id

    @default_signature_policy_id.setter
    def default_signature_policy_id(self, value):
        self.__set_default_signature_policy_id(value)

    def __set_default_signature_policy_id(self, value):
        if value is None:
            raise Exception('The provided "default_signature_policy_id" is not '
                            'valid')
        self.__default_signature_policy_id = value

    # endregion

    # region "acceptable_explicit_policies" accessors

    @property
    def acceptable_explicit_policies(self):
        return self.__get_acceptable_explicit_policies()

    def __get_acceptable_explicit_policies(self):
        return self.__acceptable_explicit_policies

    @acceptable_explicit_policies.setter
    def acceptable_explicit_policies(self, value):
        self.__set_acceptable_explicit_policies(value)

    def __set_acceptable_explicit_policies(self, value):
        if value is None:
            raise Exception('The provided "acceptable_explicit_policies" is not'
                            ' valid')
        self.__acceptable_explicit_policies = value

    # endregion

    # region "ignore_revocation_status_unknown" accessors

    @property
    def ignore_revocation_status_unknown(self):
        return self.__get_ignore_revocation_status_unknown()

    def __get_ignore_revocation_status_unknown(self):
        return self.__ignore_revocation_status_unknown

    @ignore_revocation_status_unknown.setter
    def ignore_revocation_status_unknown(self, value):
        self.__set_ignore_revocation_status_unknown(value)

    def __set_ignore_revocation_status_unknown(self, value):
        if value is None:
            raise Exception('The provided "ignore_revocation_status_unknown" is'
                            ' not valid')
        self.__ignore_revocation_status_unknown = value

    # endregion

    def _fill_request(self, request):
        request['validate'] = self.__validate
        request['defaultSignaturePolicyId'] = self.__default_signature_policy_id
        request['securityContextId'] = self.__security_context_id
        request['ignoreRevocationStatusUnknown'] = \
            self.__ignore_revocation_status_unknown
        if self.__acceptable_explicit_policies is not None:
            policy_ids = [policy.id for policy in
                          self.__acceptable_explicit_policies.policies]
            request['acceptableExplicitPolicies'] = policy_ids
        request['file'] = \
            self._signature_file.upload_or_reference(self._client)


__all__ = ['SignatureExplorer']



from abc import ABCMeta
from abc import abstractmethod

from .file_reference import FileReference
from .namespace_manager import NamespaceManager
from .signature_starter import SignatureStarter


class XmlSignatureStarter(SignatureStarter):
    __metaclass__ = ABCMeta

    def __init__(self, client):
        super(XmlSignatureStarter, self).__init__(client)
        self._xml_to_sign = None
        self._signature_element_id = None
        self._signature_element_location_xpath = None
        self._signature_element_location_nsm = NamespaceManager()
        self._signature_element_location_insertion_option = None

    # region "xml_to_sign" accessors

    @property
    def xml_to_sign(self):
        return self.__get_xml_to_sign()

    def __get_xml_to_sign(self):
        return self._xml_to_sign.file_desc

    @xml_to_sign.setter
    def xml_to_sign(self, value):
        self.__set_xml_to_sign(value)

    def __set_xml_to_sign(self, value):
        if value is None:
            raise Exception('The provided "xml_to_sign" is not valid')
        self._xml_to_sign = FileReference.from_file(value)

    # endregion

    # region "xml_to_sign_path" accessors

    @property
    def xml_to_sign_path(self):
        return self.__get_xml_to_sign_path()

    def __get_xml_to_sign_path(self):
        return self._xml_to_sign.path

    @xml_to_sign_path.setter
    def xml_to_sign_path(self, value):
        self.__set_xml_to_sign_path(value)

    def __set_xml_to_sign_path(self, value):
        if value is None:
            raise Exception('The provided "xml_to_sign_path" is not valid')
        self._xml_to_sign = FileReference.from_path(value)

    def set_xml_to_sign_from_path(self, path):
        self.__set_xml_to_sign_path(path)

    def set_xml_to_sign(self, path):
        self.set_xml_to_sign_from_path(path)

    # endregion

    # region "xml_to_sign_base64" accessors

    @property
    def xml_to_sign_base64(self):
        return self.__get_xml_to_sign_base64()

    def __get_xml_to_sign_base64(self):
        return self._xml_to_sign.content_base64

    @xml_to_sign_base64.setter
    def xml_to_sign_base64(self, value):
        self.__set_xml_to_sign_base64(value)

    def __set_xml_to_sign_base64(self, value):
        if value is None:
            raise Exception('The provided "xml_to_sign_base64" is not valid')
        self._xml_to_sign = FileReference.from_content_base64(value)

    def set_xml_to_sign_from_content_base64(self, content_base64):
        self.__set_xml_to_sign_base64(content_base64)

    # endregion

    # region "xml_to_sign_content" accessors

    @property
    def xml_to_sign_content(self):
        return self.__get_xml_to_sign_content()

    def __get_xml_to_sign_content(self):
        return self._xml_to_sign.content_raw

    @xml_to_sign_content.setter
    def xml_to_sign_content(self, value):
        self.__set_xml_to_sign_content(value)

    def __set_xml_to_sign_content(self, value):
        if value is None:
            raise Exception('The provided "xml_to_sign_content" is not valid')
        self._xml_to_sign = FileReference.from_content_raw(value)

    def set_xml_to_sign_from_content_raw(self, content_raw):
        self.__set_xml_to_sign_content(content_raw)

    def set_xml_to_sign_content(self, content_raw):
        self.set_xml_to_sign_from_content_raw(content_raw)

    # endregion

    @property
    def signature_element_id(self):
        return self._signature_element_id

    @signature_element_id.setter
    def signature_element_id(self, value):
        self._signature_element_id = value

    @property
    def signature_element_location_xpath(self):
        return self._signature_element_location_xpath

    @property
    def signature_element_location_nsm(self):
        return self._signature_element_location_nsm

    @property
    def signature_element_location_insertion_option(self):
        return self._signature_element_location_insertion_option

    def set_signature_element_location(self,
                                       xpath,
                                       insertion_option,
                                       namespace_manager):
        self._signature_element_location_xpath = xpath
        self._signature_element_location_insertion_option = insertion_option
        self._signature_element_location_nsm = namespace_manager

    def _verify_common_parameters(self, is_with_web_pki=False):
        if not is_with_web_pki and self._signer_certificate is None:
            raise Exception('The certificate was not set')

        if self._xml_to_sign is None:
            raise Exception('The XML was not set')

        if self._signature_policy_id is None:
            raise Exception('The signature policy was not set')

    def _fill_request(self, request):
        request['xml'] = self._xml_to_sign.content_base64
        request['certificate'] = self._signer_certificate
        request['signaturePolicyId'] = self._signature_policy_id
        request['securityContextId'] = self._security_context_id
        request['ignoreRevocationStatusUnknown'] = \
            self._ignore_revocation_status_unknown
        request['callbackArgument'] = self._callback_argument
        if self._signature_element_location_xpath is not None and \
                self._signature_element_location_insertion_option is not None:
            request['signatureElementLocation'] = {
                'xPath': self._signature_element_location_xpath,
                'insertionOption':
                    self._signature_element_location_insertion_option,
                'namespaces': self._signature_element_location_nsm.namespaces
                              if self._signature_element_location_nsm is not
                              None else None
            }

        return request

    @abstractmethod
    def start(self):
        pass

    @abstractmethod
    def start_with_web_pki(self):
        pass


__all__ = ['XmlSignatureStarter']



from abc import ABCMeta
from abc import abstractmethod

from .signature_result import SignatureResult
from .utils import _base64_encode_string


class SignatureFinisher(object):
    __metaclass__ = ABCMeta

    def __init__(self, client):
        self._client = client
        self._token = None
        self._signature = None

        self.__force_blob_result = None

    # region "client" accessors

    @property
    def client(self):
        return self.__get_client()

    def __get_client(self):
        return self._client

    @client.setter
    def client(self, value):
        self.__set_client(value)

    def __set_client(self, value):
        if value is None:
            raise Exception('The provided "client" is not valid')
        self._client = value

    # endregion

    # region "token" accessors

    @property
    def token(self):
        return self.__get_token()

    def __get_token(self):
        return self._token

    @token.setter
    def token(self, value):
        self.__set_token(value)

    def __set_token(self, value):
        if value is None:
            raise Exception('The provided "token" is not valid')
        self._token = value

    # endregion

    # region "signature" accessors

    @property
    def signature(self):
        return self.__get_signature()

    def __get_signature(self):
        return self._signature

    @signature.setter
    def signature(self, value):
        self.__set_signature(value)

    def __set_signature(self, value):
        if value is None:
            raise Exception('The provided "signature" is not valid')
        self._signature = value

    # endregion

    # region "force_blob_result" accessors

    @property
    def force_blob_result(self):
        return self.__get_force_blob_result()

    def __get_force_blob_result(self):
        return self.__force_blob_result

    @force_blob_result.setter
    def force_blob_result(self, value):
        self.__set_force_blob_result(value)

    def __set_force_blob_result(self, value):
        if value is None:
            raise Exception('The provided "force_blob_result" is not valid')
        self.__force_blob_result = value

    # endregion

    def finish(self):
        if self._token is None:
            raise Exception('The token was not set')

        self._check_compatibility()

        request = {
            'signature': _base64_encode_string(self._signature) if
                         self._signature is not None else None,
            'forceBlobResult': self.__force_blob_result or False
        }
        response = self._client.post(self._get_api_route(), request)
        return SignatureResult(self.client,
                               response.get('signatureFile'),
                               response.get('certificate'),
                               response.get('callbackArgument'))

    @abstractmethod
    def _check_compatibility(self):
        pass

    @abstractmethod
    def _get_api_route(self):
        pass


__all__ = ['SignatureFinisher']



from .pdf_mark_page_options import PdfMarkPageOptions
from .color import Color


class PdfMark(object):

    def __init__(self):
        self.__container = None
        self.__border_width = 0.0
        self.__border_color = Color.BLACK
        self.__background_color = Color.TRANSPARENT
        self.__elements = []
        self.__page_option = PdfMarkPageOptions.ALL_PAGES
        self.__page_option_number = None

    @property
    def container(self):
        return self.__container

    @container.setter
    def container(self, value):
        self.__container = value

    @property
    def border_width(self):
        return self.__border_width

    @border_width.setter
    def border_width(self, value):
        self.__border_width = value

    @property
    def border_color(self):
        return self.__border_color

    @border_color.setter
    def border_color(self, value):
        self.__border_color = value

    @property
    def background_color(self):
        return self.__background_color

    @background_color.setter
    def background_color(self, value):
        self.__background_color = value

    @property
    def elements(self):
        return self.__elements

    @elements.setter
    def elements(self, value):
        self.__elements = value

    @property
    def page_option(self):
        return self.__page_option

    @page_option.setter
    def page_option(self, value):
        self.__page_option = value

    @property
    def page_option_number(self):
        return self.__page_option_number

    @page_option_number.setter
    def page_option_number(self, value):
        self.__page_option_number = value

    # region Fluent API

    def on_container(self, container):
        self.__container = container
        return self

    def with_border_width(self, border_width):
        self.__border_width = border_width
        return self

    def on_all_pages(self):
        self.__page_option = PdfMarkPageOptions.ALL_PAGES
        return self

    def on_new_page(self):
        self.__page_option = PdfMarkPageOptions.NEW_PAGE
        return self

    def on_single_page(self, page_number):
        self.__page_option = PdfMarkPageOptions.SINGLE_PAGE
        self.__page_option_number = page_number
        return self

    def on_single_page_from_end(self, page_number):
        self.__page_option = PdfMarkPageOptions.SINGLE_PAGE_FROM_END
        self.__page_option_number = page_number
        return self

    def add_element(self, element):
        self.__elements.append(element)
        return self

    def with_border_color(self, border_color):
        self.__border_color = border_color
        return self

    def with_background_color(self, background_color):
        self.__background_color = background_color
        return self

    # endregion

    def to_model(self):
        return {
            'container': self.__container.to_model()
            if self.__container is not None else None,
            'backgroundColor': self.__background_color.to_model()
            if self.__background_color is not None else None,
            'borderColor': self.__border_color.to_model()
            if self.__border_color is not None else None,
            'borderWidth': self.__border_width,
            'pageOption': self.__page_option,
            'pageOptionNumber': self.__page_option_number,
            'elements': [e.to_model() for e in self.__elements]
        }


__all__ = ['PdfMark']



from .pdf_text_section import PdfTextSection
from .pades_horizontal_align import PadesHorizontalAlign
from .pdf_mark_element_type import PdfMarkElementType
from .pdf_mark_element import PdfMarkElement


class PdfMarkTextElement(PdfMarkElement):

    def __init__(self, relative_container=None, text_sections=None):
        super(PdfMarkTextElement, self).__init__(PdfMarkElementType.TEXT,
                                                 relative_container)
        self.__text_sections = text_sections \
            if text_sections is not None else []
        self.__align = PadesHorizontalAlign.LEFT

    # region FluentAPI

    def align_text_left(self):
        self.__align = PadesHorizontalAlign.LEFT
        return self

    def align_text_right(self):
        self.__align = PadesHorizontalAlign.RIGHT
        return self

    def align_text_center(self):
        self.__align = PadesHorizontalAlign.CENTER
        return self

    def add_section_from_text(self, section):
        self.__text_sections.append(PdfTextSection(section))
        return self

    def add_section(self, section):
        self.__text_sections.append(section)
        return self

    # endregion

    @property
    def text_sections(self):
        return self.__text_sections

    @text_sections.setter
    def text_sections(self, value):
        self.__text_sections = value

    @property
    def align(self):
        return self.__align

    @align.setter
    def align(self, value):
        self.__align = value

    def to_model(self):
        model = super(PdfMarkTextElement, self).to_model()
        model['align'] = self.__align
        model['textSections'] = [s.to_model() for s in self.__text_sections]
        return model


__all__ = ['PdfMarkTextElement']



class Color(object):
    TRANSPARENT = None
    WHITE = None
    LIGHT_GRAY = None
    GRAY = None
    DARK_GRAY = None
    BLACK = None
    RED = None
    PINK = None
    ORANGE = None
    YELLOW = None
    GREEN = None
    MAGENTA = None
    CYAN = None
    BLUE = None

    def __init__(self, red, green, blue, alpha=100):
        self.__red = red
        self.__green = green
        self.__blue = blue
        self.__alpha = alpha

    @staticmethod
    def from_rgb(red, green, blue):
        return Color(red, green, blue)

    @staticmethod
    def from_rgba(red, green, blue, alpha):
        return Color(red, green, blue, alpha)

    @property
    def red(self):
        return self.__red

    @red.setter
    def red(self, value):
        self.__red = value

    @property
    def green(self):
        return self.__green

    @green.setter
    def green(self, value):
        self.__green = value

    @property
    def blue(self):
        return self.__blue

    @blue.setter
    def blue(self, value):
        self.__blue = value

    @property
    def alpha(self):
        return self.__alpha

    @alpha.setter
    def alpha(self, value):
        self.__alpha = value

    def to_model(self):
        return {
            'alpha': self.__alpha / 2.55,
            'red': self.__red,
            'green': self.__green,
            'blue': self.__blue
        }


Color.TRANSPARENT = Color(255, 255, 255, 0)
Color.WHITE = Color(255, 255, 255)
Color.LIGHT_GRAY = Color(192, 192, 192)
Color.GRAY = Color(128, 128, 128)
Color.DARK_GRAY = Color(64, 64, 64)
Color.BLACK = Color(0, 0, 0)
Color.RED = Color(255, 0, 0)
Color.PINK = Color(255, 175, 175)
Color.ORANGE = Color(255, 200, 0)
Color.YELLOW = Color(255, 255, 0)
Color.GREEN = Color(0, 255, 0)
Color.MAGENTA = Color(255, 0, 255)
Color.CYAN = Color(0, 255, 255)
Color.BLUE = Color(0, 0, 255)

__all__ = ['Color']



from .xml_signature_starter import XmlSignatureStarter
from .signature_start_result import SignatureStartResult


class FullXmlSignatureStarter(XmlSignatureStarter):

    def start(self):
        return self.__start_common()

    def start_with_web_pki(self):
        return self.__start_common(True)

    def start_with_webpki(self):
        return self.start_with_web_pki()

    def __start_common(self, is_with_web_pki=False):
        request = self.__get_start_common_request(is_with_web_pki)
        response = self._client.post('Api/XmlSignatures/FullXmlSignature',
                                     request)
        return SignatureStartResult(response)

    def __get_start_common_request(self, is_with_web_pki=False):
        self._verify_common_parameters(is_with_web_pki)
        request = {}
        self._fill_request(request)
        return request


__all__ = ['FullXmlSignatureStarter']



from .xml_signature_starter import XmlSignatureStarter
from .signature_start_result import SignatureStartResult


class OnlineResourceXmlSignatureStarter(XmlSignatureStarter):

    def __init__(self, client):
        super(OnlineResourceXmlSignatureStarter, self).__init__(client)
        self.__resource_uri = None

    # region "resource_uri" accessors

    @property
    def resource_uri(self):
        return self.__get_resource_uri()

    def __get_resource_uri(self):
        return self.__resource_uri

    @resource_uri.setter
    def resource_uri(self, value):
        self.__set_resource_uri(value)

    def __set_resource_uri(self, value):
        if value is None:
            raise Exception('The provided "resource_uri" is not valid')
        self.__resource_uri = value

    # endregion

    def start(self):
        return self.__start_common()

    def start_with_web_pki(self):
        return self.__start_common(True)

    def start_with_webpki(self):
        return self.start_with_web_pki()

    def __start_common(self, is_with_web_pki=False):
        request = self.__get_start_common_request(is_with_web_pki)
        response = \
            self._client.post('Api/XmlSignatures/OnlineResourceXmlSignature',
                              request)
        return SignatureStartResult(response)

    def __get_start_common_request(self, is_with_web_pki=False):
        self._verify_common_parameters(is_with_web_pki)
        if self.__resource_uri is None:
            raise Exception('The online resource URI to sign was not set')
        request = {}
        self._fill_request(request)
        request['resourceToSignUri'] = self.__resource_uri
        return request


__all__ = ['OnlineResourceXmlSignatureStarter']



import base64

from six import BytesIO

from .utils import _base64_decode
from .utils import _base64_encode_string
from .utils import _copy_stream
from .utils import _get_raw_stream


class FileResult(object):

    def __init__(self, client, file_model):
        self.__client = client
        self.__file = file_model

    @property
    def client(self):
        return self.__client

    @client.setter
    def client(self, value):
        self.__client = value

    def open(self, buff_size=4096):
        content_base64 = self.__file.get('content')
        content_raw = _base64_decode(content_base64)\
            if content_base64 is not None else None
        if content_raw is not None:
            stream = _get_raw_stream(content_raw)
        else:
            stream = BytesIO()
            for chunk in self.__client.get_raw(self.__file.get('url'))\
                    .iter_content(buff_size):
                stream.write(chunk)
            stream.seek(0, 0)
        return stream

    def write_to(self, f_out, offset=0, from_where=0, buff_size=4096):
        content_base64 = self.__file.get('content')
        content_raw = _base64_decode(content_base64)\
            if content_base64 is not None else None
        if content_raw is not None:
            f_in = self.open()
            _copy_stream(f_in, f_out, offset, from_where, buff_size)
            f_in.close()
        else:
            f_out.seek(offset, from_where)
            for chunk in self.__client.get_raw(self.__file.get('url'))\
                    .iter_content(buff_size):
                f_out.write(chunk)

    def write_to_file(self, path, offset=0, from_where=0, buff_size=4096):
        content_base64 = self.__file.get('content')
        content_raw = _base64_decode(content_base64)\
            if content_base64 is not None else None
        with open(path, 'wb') as f_out:
            if content_raw is not None:
                f_in = self.open()
                _copy_stream(f_in, f_out, offset, from_where, buff_size)
                f_in.close()
            else:
                for chunk in self.__client.get_raw(self.__file.get('url'))\
                        .iter_content(buff_size):
                    f_out.write(chunk)

    # region "content" accessors

    @property
    def content(self):
        return self.__get_content()

    def __get_content(self):
        buffer = BytesIO()
        self.write_to(buffer)
        buffer.seek(0, 0)
        return buffer.read()

    @content.setter
    def content(self, value):
        self.__set_content(value)

    def __set_content(self, value):
        if value is None:
            raise Exception('The provided "content" is not valid')
        if self.__file is None:
            self.__file = {
                'content': None,
                'mimeType': None,
                'blobToken': None,
                'url': None
            }
        self.__file['content'] = base64.b64encode(value)

    # endregion

    # region "content_base64" accessors

    @property
    def content_base64(self):
        return self.__get_content_base64()

    def __get_content_base64(self):
        buffer = BytesIO()
        self.write_to(buffer)
        buffer.seek(0, 0)
        return _base64_encode_string(buffer.read())

    @content_base64.setter
    def content_base64(self, value):
        self.__set_content_base64(value)

    def __set_content_base64(self, value):
        if value is None:
            raise Exception('The provided "content_base64" is not valid')
        if self.__file is None:
            self.__file = {
                'content': None,
                'mimeType': None,
                'blobToken': None,
                'url': None
            }
        self.__file['content'] = value

    # endregion


__all__ = ['FileResult']



class Name(object):

    def __init__(self, model):
        self._common_name = model.get('commonName', None)
        self._country = model.get('country', None)
        self._dn_qualifier = model.get('dnQualifier', None)
        self._email_address = model.get('emailAddress', None)
        self._generation_qualifier = model.get('generationQualifier', None)
        self._given_name = model.get('givenName', None)
        self._initials = model.get('initials', None)
        self._locality = model.get('locality', None)
        self._organization = model.get('organization', None)
        self._organization_unit = model.get('organizationUnit', None)
        self._pseudonym = model.get('pseudonym', None)
        self._serial_number = model.get('serialNumber', None)
        self._state_name = model.get('stateName', None)
        self._surname = model.get('surname', None)
        self._title = model.get('title', None)

    @property
    def common_name(self):
        return self._common_name

    @common_name.setter
    def common_name(self, value):
        self._common_name = value

    @property
    def country(self):
        return self._country

    @country.setter
    def country(self, value):
        self._country = value

    @property
    def dn_qualifier(self):
        return self._dn_qualifier

    @dn_qualifier.setter
    def dn_qualifier(self, value):
        self._dn_qualifier = value

    @property
    def email_address(self):
        return self._email_address

    @email_address.setter
    def email_address(self, value):
        self._email_address = value

    @property
    def generation_qualifier(self):
        return self._generation_qualifier

    @generation_qualifier.setter
    def generation_qualifier(self, value):
        self._generation_qualifier = value

    @property
    def given_name(self):
        return self._given_name

    @given_name.setter
    def given_name(self, value):
        self._given_name = value

    @property
    def initials(self):
        return self._initials

    @initials.setter
    def initials(self, value):
        self._initials = value

    @property
    def locality(self):
        return self._locality

    @locality.setter
    def locality(self, value):
        self._locality = value

    @property
    def organization(self):
        return self._organization

    @organization.setter
    def organization(self, value):
        self._organization = value

    @property
    def organization_unit(self):
        return self._organization_unit

    @organization_unit.setter
    def organization_unit(self, value):
        self._organization_unit = value

    @property
    def pseudonym(self):
        return self._pseudonym

    @pseudonym.setter
    def pseudonym(self, value):
        self._pseudonym = value

    @property
    def serial_number(self):
        return self._serial_number

    @serial_number.setter
    def serial_number(self, value):
        self._serial_number = value

    @property
    def state_name(self):
        return self._state_name

    @state_name.setter
    def state_name(self, value):
        self._state_name = value

    @property
    def surname(self):
        return self._surname

    @surname.setter
    def surname(self, value):
        self._surname = value

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, value):
        self._title = value


__all__ = ['Name']



class Enum(object):

    def __init__(self, value):
        self.__value = value

    def __eq__(self, instance_or_value):
        if instance_or_value is None:
            return False

        if type(instance_or_value) is Enum:
            return self.__value == instance_or_value.value
        return self.__value == instance_or_value

    # region "value" accessors

    @property
    def value(self):
        """

        The definition of property "value", which contains the value of the
        enumeration.

        """
        return self.__get_value()

    def __get_value(self):
        return self.__value

    @value.setter
    def value(self, val):
        self.__set_value(val)

    def __set_value(self, val):
        if val is None:
            raise Exception('The provided "value" is not valid')
        self.__value = val

    # endregion


__all__ = ['Enum']



class PdfMarkElement(object):

    def __init__(self, element_type, relative_container=None):
        self.__element_type = element_type
        self.__relative_container = relative_container
        self.__rotation = 0
        self.__opacity = 100

    @property
    def element_type(self):
        return self.__element_type

    @element_type.setter
    def element_type(self, value):
        self.__element_type = value

    @property
    def relative_container(self):
        return self.__relative_container

    @relative_container.setter
    def relative_container(self, value):
        self.__relative_container = value

    @property
    def rotation(self):
        return self.__rotation

    @rotation.setter
    def rotation(self, value):
        self.__rotation = value

    @property
    def opacity(self):
        return self.__opacity

    @opacity.setter
    def opacity(self, value):
        self.__opacity = value

    # region FluentAPI

    def on_container(self, relative_container):
        self.__relative_container = relative_container
        return self

    def with_rotation(self, rotation):
        self.__rotation = rotation
        return self

    def rotate_90_clockwise(self):
        self.__rotation = 270
        return self

    def rotate_90_counter_clockwise(self):
        self.__rotation = 90
        return self

    def rotate_180(self):
        self.__rotation = 180
        return self

    def with_opacity(self, opacity):
        self.__opacity = opacity
        return self

    # endregion

    def to_model(self):
        return {
            'elementType': self.__element_type,
            'relativeContainer': self.__relative_container.to_model() if
            self.__relative_container is not None else None,
            'rotation': self.__rotation,
            'opacity': self.__opacity
        }


__all__ = ['PdfMarkElement']



from .authentication_result import AuthenticationResult


class Authentication(object):

    def __init__(self, client):
        self._client = client

    @property
    def client(self):
        return self._client

    @client.setter
    def client(self, value):
        self._client = value

    def start(self):
        response = self.client.get('Api/Authentication')
        nonce = response.get('nonce', None)
        return nonce

    def start_with_web_pki(self,
                           security_context_id,
                           ignore_revocation_status_unknown=False):
        request = {
            'securityContextId': security_context_id,
            'ignoreRevocationStatusUnknown': ignore_revocation_status_unknown
        }
        response = self._client.post('Api/Authentications', request)
        token = response.get('token', None)
        return token

    def start_with_webpki(self,
                          security_context_id,
                          ignore_revocation_status_unknown=False):
        return self.start_with_web_pki(security_context_id,
                                       ignore_revocation_status_unknown)

    def complete(self,
                 nonce,
                 cert_content,
                 signature,
                 security_content_id,
                 ignore_revocation_status_unknown=False):
        request = {
            'nonce': nonce,
            'certificate': cert_content,
            'signature': signature,
            'securityContextId': security_content_id,
            'ignoreRevocationStatusUnknown': ignore_revocation_status_unknown
        }
        response = self._client.post('Api/Authentication', request)
        return AuthenticationResult(response)

    def complete_with_web_pki(self, token):
        response = self._client.post('Api/Authentications/%s/Finalize' % token)
        return AuthenticationResult(response)

    def complete_with_webpki(self, token):
        return self.complete_with_web_pki(token)


__all__ = ['Authentication']



from .rest_base_error import RestBaseError


class RestPkiError(RestBaseError):

    def __init__(self, verb, url, error_code, detail):
        message = "REST PKI action %s %s error: %s" % (verb, url, error_code)
        if detail and len(detail) > 0:
            message += " (%s)" % detail
        RestBaseError.__init__(self, __name__, message, verb, url)

        self._error_code = error_code
        self._detail = detail

    @property
    def error_code(self):
        return self._error_code

    @error_code.setter
    def error_code(self, value):
        self._error_code = value

    @property
    def detail(self):
        return self._detail

    @detail.setter
    def detail(self, value):
        self._detail = value


__all__ = ['RestPkiError']



class PaperSizes:
    CUSTOM = 'Custom'
    A0 = 'A0'
    A1 = 'A1'
    A2 = 'A2'
    A3 = 'A3'
    A4 = 'A4'
    A5 = 'A5'
    A6 = 'A6'
    A7 = 'A7'
    A8 = 'A8'
    LETTER = 'Letter'
    LEGAL = 'Legal'
    LEDGER = 'Ledger'


__all__ = ['PaperSizes']



from .pades_visual_rectangle import PadesVisualRectangle


class PdfContainerDefinition(object):

    class VarWidthAndHeight(object):

        def __init__(self, container):
            self.__container = container

        def margins(self,
                    top_margin,
                    right_margin=None,
                    bottom_margin=None,
                    left_margin=None):
            self.__container.top = top_margin
            self.__container.right = right_margin \
                if right_margin is not None else top_margin
            self.__container.bottom = bottom_margin \
                if bottom_margin is not None else top_margin
            self.__container.left = left_margin \
                if left_margin is not None else self.__container.right
            return self.__container

    class HeightDefinedVarWidth(object):

        def __init__(self, container):
            self.__container = container

        def margins(self, left_margin, right_margin=None):
            self.__container.left = left_margin
            self.__container.right = right_margin \
                if right_margin is not None else left_margin
            return self.__container

    class HeightDefinedFixedWidth(object):

        def __init__(self, container):
            self.__container = container

        def anchor_left(self, margin=0.0):
            self.__container.left = margin
            return self.__container

        def anchor_right(self, margin=0.0):
            self.__container.right = margin
            return self.__container

        def center(self):
            return self.__container

    class WidthDefinedVarHeight(object):

        def __init__(self, container):
            self.__container = container

        def margins(self, top_margin, bottom_margin=None):
            self.__container.top = top_margin
            self.__container.bottom = bottom_margin \
                if bottom_margin is not None else top_margin
            return self.__container

    class WidthDefinedFixedHeight(object):

        def __init__(self, container):
            self.__container = container

        def anchor_top(self, margin=0.0):
            self.__container.top = margin
            return self.__container

        def anchor_bottom(self, margin=0.0):
            self.__container.bottom = margin
            return self.__container

        def center(self):
            return self.__container

    class HeightDefined(object):

        def __init__(self, container):
            self.__container = container

        def width(self, value):
            self.__container.width = value
            return PdfContainerDefinition.\
                HeightDefinedFixedWidth(self.__container)

        def var_width(self):
            return PdfContainerDefinition.\
                HeightDefinedVarWidth(self.__container)

        def full_width(self):
            self.__container.left = 0.0
            self.__container.right = 0.0
            return self.__container

    class WidthDefined(object):

        def __init__(self, container):
            self.__container = container

        def height(self, value):
            self.__container.height = value
            return PdfContainerDefinition.\
                WidthDefinedFixedHeight(self.__container)

        def var_height(self):
            return PdfContainerDefinition.\
                WidthDefinedVarHeight(self.__container)

        def full_height(self):
            self.__container.top = 0.0
            self.__container.bottom = 0.0
            return self.__container

    class VarHeight(object):

        def __init__(self, container):
            self.__container = container

        def margins(self, top_margin, bottom_margin=None):
            self.__container.top = top_margin
            self.__container.bottom = bottom_margin \
                if bottom_margin is not None else top_margin
            return PdfContainerDefinition.HeightDefined(self.__container)

    class FixedHeight(object):

        def __init__(self, container):
            self.__container = container

        def anchor_top(self, margin=0.0):
            self.__container.top = margin
            return PdfContainerDefinition.HeightDefined(self.__container)

        def anchor_bottom(self, margin=0.0):
            self.__container.bottom = margin
            return PdfContainerDefinition.HeightDefined(self.__container)

        def center(self):
            return PdfContainerDefinition.HeightDefined(self.__container)

    class VarWidth(object):

        def __init__(self, container):
            self.__container = container

        def margins(self, left_margin, right_margin=None):
            self.__container.left = left_margin
            self.__container.right = right_margin \
                if right_margin is not None else left_margin
            return PdfContainerDefinition.WidthDefined(self.__container)

    class FixedWidth(object):

        def __init__(self, container):
            self.__container = container

        def anchor_left(self, margin=0.0):
            self.__container.left = margin
            return PdfContainerDefinition.WidthDefined(self.__container)

        def anchor_right(self, margin=0.0):
            self.__container.right = margin
            return PdfContainerDefinition.WidthDefined(self.__container)

        def center(self):
            return PdfContainerDefinition.WidthDefined(self.__container)

    class Initial(object):

        def __init__(self):
            self.__container = PadesVisualRectangle()

        def width(self, value):
            self.__container.width = value
            return PdfContainerDefinition.FixedWidth(self.__container)

        def var_width(self):
            return PdfContainerDefinition.VarWidth(self.__container)

        def full_width(self):
            self.__container.left = 0.0
            self.__container.right = 0.0
            return PdfContainerDefinition.WidthDefined(self.__container)

        def height(self, value):
            self.__container.height = value
            return PdfContainerDefinition.FixedHeight(self.__container)

        def var_height(self):
            return PdfContainerDefinition.VarHeight(self.__container)

        def full_height(self):
            self.__container.top = 0.0
            self.__container.bottom = 0.0
            return PdfContainerDefinition.HeightDefined(self.__container)

        def var_width_and_height(self):
            return PdfContainerDefinition.VarWidthAndHeight(self.__container)

        def full(self):
            self.__container.top = 0.0
            self.__container.right = 0.0
            self.__container.bottom = 0.0
            self.__container.left = 0.0
            return self.__container


__all__ = ['PdfContainerDefinition']



from .rest_base_error import RestBaseError


class RestError(RestBaseError):

    def __init__(self, verb, url, status_code, error_message=None):
        message = "REST action %s %s returned HTTP error %s" % \
                  (verb, url, status_code)
        if error_message and len(error_message) > 0:
            message += ": %s" % error_message
        RestBaseError.__init__(self, __name__, message, verb, url)

        self._status_code = status_code
        self._error_message = error_message

    @property
    def status_code(self):
        return self._status_code

    @status_code.setter
    def status_code(self, value):
        self._status_code = value

    @property
    def error_message(self):
        return self._error_message

    @error_message.setter
    def error_message(self, value):
        self._error_message = value


__all__ = ['RestError']



class BlobReference(object):

    def __init__(self, token):
        self.__token = token

    # region "token" accessors

    @property
    def token(self):
        return self.__get_token()

    def __get_token(self):
        return self.__token

    @token.setter
    def token(self, value):
        self.__set_token(value)

    def __set_token(self, value):
        if value is None:
            raise Exception('The provided "token" is not valid')
        self.__token = value

    # endregion


__all__ = ['BlobReference']



class PadesVisualPositioningPresets(object):
    cached_presets = dict()

    @staticmethod
    def get_footnote(client, page_number=None, rows=None):

        url_segment = 'Footnote'
        if page_number:
            url_segment += '?pageNumber=%s' % page_number

        if rows:
            url_segment += '&' if page_number else '?'
            url_segment += 'rows=%s' % rows

        return PadesVisualPositioningPresets.__get_preset(client, url_segment)

    @staticmethod
    def get_new_page(client):
        return PadesVisualPositioningPresets.__get_preset(client,
                                                          'NewPage')

    @staticmethod
    def __get_preset(client, url_segment):
        if url_segment in PadesVisualPositioningPresets.cached_presets:
            return PadesVisualPositioningPresets.cached_presets[url_segment]

        preset = client.get(
            'Api/PadesVisualPositioningPresets/%s' % url_segment)

        PadesVisualPositioningPresets.cached_presets[url_segment] = preset
        return preset


__all__ = ['PadesVisualPositioningPresets']



class StandardSignaturePolicies(object):
    PADES_BASIC = '78d20b33-014d-440e-ad07-929f05d00cdf'
    PADES_BASIC_WITH_PKI_BRAZIL_CERTS = '3fec800c-366c-49bf-82c5-2e72154e70f6'
    PADES_T_WITH_PKI_BRAZIL_CERTS = '6a39aeea-a2d0-4754-bf8c-19da15296ddb'
    PKI_BRAZIL_PADES_ADR_BASICA = '531d5012-4c0d-4b6f-89e8-ebdcc605d7c2'
    PKI_BRAZIL_PADES_ADR_TEMPO = '10f0d9a5-a0a9-42e9-9523-e181ce05a25b'

    CADES_BES = 'a4522485-c9e5-46c3-950b-0d6e951e17d1'
    CADES_BES_WITH_SIGNING_TIME_AND_NO_CRLS =\
        '8108539d-c137-4f45-a1f2-de5305bc0a37'
    PKI_BRAZIL_CADES_ADR_BASICA = '3ddd8001-1672-4eb5-a4a2-6e32b17ddc46'
    PKI_BRAZIL_CADES_ADR_TEMPO = 'a5332ad1-d105-447c-a4bb-b5d02177e439'
    PKI_BRAZIL_CADES_ADR_COMPLETA = '30d881e7-924a-4a14-b5cc-d5a1717d92f6'

    XML_DSIG_BASIC = '2bb5d8c9-49ba-4c62-8104-8141f6459d08'
    XADES_BES = '1beba282-d1b6-4458-8e46-bd8ad6800b54'
    PKI_BRAZIL_XADES_ADR_BASICA = '1cf5db62-58b6-40ba-88a3-d41bada9b621'
    PKI_BRAZIL_XADES_ADR_TEMPO = '5aa2e0af-5269-43b0-8d45-f4ef52921f04'
    NFE_PADRAO_NACIONAL = 'a3c24251-d43a-4ba4-b25d-ee8e2ab24f06'

    def __init__(self):
        pass


__all__ = ['StandardSignaturePolicies']



from six import BytesIO
from six import text_type

from .utils import _base64_decode
from .utils import _base64_encode_string


class FileReference(object):

    def __init__(self):
        self.__path = None
        self.__blob_token = None
        self.__content_base64 = None
        self.__content_raw = None
        self.__file_desc = None

    @staticmethod
    def _get_file_size(file_desc):
        file_desc.seek(0, 2)
        size = file_desc.tell()
        file_desc.seek(0, 0)
        return size

    @staticmethod
    def from_file(file_desc):
        reference = FileReference()
        reference.file_desc = file_desc
        return reference

    @staticmethod
    def from_path(path):
        reference = FileReference()
        reference.path = path
        return reference

    @staticmethod
    def from_content_raw(content_raw):
        reference = FileReference()
        reference.content_raw = content_raw
        return reference

    @staticmethod
    def from_content_base64(content_base64):
        reference = FileReference()
        reference.content_base64 = content_base64
        return reference

    @staticmethod
    def from_blob(blob):
        reference = FileReference()
        reference.blob_token = blob.token
        return reference

    @staticmethod
    def from_result(result):
        if result.file.blob_token is not None:
            reference = FileReference()
            reference.blob_token = result.file.blob_token
            return reference
        return FileReference.from_content_raw(result.file.content)

    # region "path" accessors

    @property
    def path(self):
        return self.__get_path()

    def __get_path(self):
        return self.__path

    @path.setter
    def path(self, value):
        self.__set_path(value)

    def __set_path(self, value):
        if value is None:
            raise Exception('The provided "path" is not valid')
        self.__path = value

    # endregion

    # region "content_base64" accessors

    @property
    def content_base64(self):
        return self.__get_content_base64()

    def __get_content_base64(self):
        if self.__content_base64 is not None:
            return self.__content_base64
        if self.__content_raw is not None:
            return _base64_encode_string(self.__content_raw)
        content_raw = self.__open_or_use_existing_stream(lambda s: s.read())
        return _base64_encode_string(content_raw)

    @content_base64.setter
    def content_base64(self, value):
        self.__set_content_base64(value)

    def __set_content_base64(self, value):
        if value is None:
            raise Exception('The provided "content_base64" is not valid')
        self.__content_base64 = value

    # endregion

    # region "content_raw" accessors

    @property
    def content_raw(self):
        return self.__get_content_raw()

    def __get_content_raw(self):
        if self.__content_raw is not None:
            return self.__content_raw
        if self.__content_base64 is not None:
            return _base64_encode_string(self.__content_base64)
        return self.__open_or_use_existing_stream(lambda s: s.read())

    @content_raw.setter
    def content_raw(self, value):
        self.__set_content_raw(value)

    def __set_content_raw(self, value):
        if value is None:
            raise Exception('The provided "content_raw" is not valid')
        self.__content_raw = value

    # endregion

    # region "blob_token" accessors

    @property
    def blob_token(self):
        return self.__get_blob_token()

    def __get_blob_token(self):
        return self.__blob_token

    @blob_token.setter
    def blob_token(self, value):
        self.__set_blob_token(value)

    def __set_blob_token(self, value):
        if value is None:
            raise Exception('The provided "blob_token" is not valid')
        self.__blob_token = value

    # endregion

    # region "file_desc" accessors

    @property
    def file_desc(self):
        return self.__get_file_desc()

    def __get_file_desc(self):
        return self.__file_desc

    @file_desc.setter
    def file_desc(self, value):
        self.__set_file_desc(value)

    def __set_file_desc(self, value):
        if value is None:
            raise Exception('The provided "file_desc" is not valid')
        self.__file_desc = value

    # endregion

    def upload_or_reference(self, client):
        file = {
            'mimeType': None,
            'content': None,
            'blobToken': None,
            'url': None
        }
        if self.__blob_token is not None:
            file['blobToken'] = self.__blob_token
        elif self.__content_base64 is not None and \
                len(_base64_decode(self.__content_base64)) < \
                client.multipart_upload_threshold:
            file['content'] = self.__content_base64
        elif self.__content_raw is not None and \
                len(self.__content_raw) < client.multipart_upload_threshold:
            file['content'] = _base64_encode_string(self.__content_raw)
        else:
            stream = self.__open_or_use_existing_stream()
            file_size = FileReference._get_file_size(stream)
            if file_size < client.multipart_upload_threshold:
                file['content'] = _base64_encode_string(stream.read())
            else:
                upload_result = client.upload_or_read(stream)
                if type(upload_result) == text_type:
                    self.__blob_token = upload_result
                    file['blobToken'] = self.__blob_token
                else:
                    file['content'] = _base64_encode_string(upload_result)
        return file

    def compute_data_hashes(self, algorithms):
        data_hashes = []
        stream = self.__open_or_use_existing_stream()
        for digest_alg in algorithms:
            stream.seek(0, 0)
            hash_func = digest_alg.get_hash_func()
            hash_func.update(stream.read())
            digest = hash_func.digest()
            data_hashes.append({
                'algorithm': digest_alg.api_model.value,
                'value': _base64_encode_string(digest),
                'hexValue': None
            })

        return data_hashes

    def __open_or_use_existing_stream(self, action=None):
        if self.__file_desc is not None:
            stream = self.__file_desc
        elif self.__path is not None:
            file_desc = open(self.__path, 'rb')
            stream = file_desc
        elif self.__content_base64 is not None:
            content_raw = _base64_decode(self.__content_base64)
            stream = BytesIO()
            stream.write(content_raw)
            stream.seek(0, 0)
        elif self.__content_raw is not None:
            stream = BytesIO()
            stream.write(self.__content_raw)
            stream.seek(0, 0)
        else:
            raise Exception('Invalid operation.')

        if action is not None:
            return action(stream)
        return stream


__all__ = ['FileReference']



from .xml_signature_starter import XmlSignatureStarter
from .signature_start_result import SignatureStartResult


class XmlElementSignatureStarter(XmlSignatureStarter):

    def __init__(self, client):
        XmlSignatureStarter.__init__(self, client)
        self.__to_sign_element_id = None
        self.__id_resolution_table = None

    @property
    def to_sign_element_id(self):
        return self.__to_sign_element_id

    @to_sign_element_id.setter
    def to_sign_element_id(self, value):
        self.__to_sign_element_id = value

    @property
    def id_resolution_table(self):
        return self.__id_resolution_table

    @id_resolution_table.setter
    def id_resolution_table(self, value):
        self.__id_resolution_table = value

    def start(self):
        return self.__start_common()

    def start_with_web_pki(self):
        return self.__start_common(True)

    def start_with_webpki(self):
        return self.start_with_web_pki()

    def __start_common(self, is_with_web_pki=False):
        request = self.__get_start_common_request(is_with_web_pki)
        response = self._client.post('Api/XmlSignatures/XmlElementSignature',
                                     request)
        return SignatureStartResult(response)

    def __get_start_common_request(self, is_with_web_pki=False):
        self._verify_common_parameters(is_with_web_pki)
        if self.__to_sign_element_id is None:
            raise Exception('The XML element id to sign was not set')
        request = {}
        self._fill_request(request)
        request['elementToSignId'] = self.__to_sign_element_id
        if self.__id_resolution_table is not None:
            request['idResolutionTable'] = self.__id_resolution_table.to_model()
        return request


__all__ = ['XmlElementSignatureStarter']



"""

Import all elements of the library to facilitate its importation from user.

"""
import restpki_client.apis
import restpki_client.authentication
import restpki_client.authentication_result
import restpki_client.blob_reference
import restpki_client.cades_signature
import restpki_client.cades_signature_finisher
import restpki_client.cades_signature_starter
import restpki_client.color
import restpki_client.detached_resource_xml_signature_starter
import restpki_client.digest_algorithm
import restpki_client.digest_algorithm_and_value
import restpki_client.file_reference
import restpki_client.file_result
import restpki_client.full_xml_signature_starter
import restpki_client.name
import restpki_client.namespace_manager
import restpki_client.oids
import restpki_client.online_resource_xml_signature_starter
import restpki_client.pades_horizontal_align
import restpki_client.pades_measurement_units
import restpki_client.pades_page_optimization
import restpki_client.pades_signature
import restpki_client.pades_signature_explorer
import restpki_client.pades_signature_finisher
import restpki_client.pades_signature_starter
import restpki_client.pades_signer_info
import restpki_client.pades_size
import restpki_client.pades_visual_positioning_presets
import restpki_client.pades_visual_rectangle
import restpki_client.page_orientations
import restpki_client.paper_sizes
import restpki_client.pdf_container_definition
import restpki_client.pdf_helper
import restpki_client.pdf_mark
import restpki_client.pdf_mark_element
import restpki_client.pdf_mark_element_type
import restpki_client.pdf_mark_image
import restpki_client.pdf_mark_image_element
import restpki_client.pdf_mark_page_options
import restpki_client.pdf_mark_qr_code_element
import restpki_client.pdf_mark_text_element
import restpki_client.pdf_marker
import restpki_client.pdf_text_section
import restpki_client.pdf_text_style
import restpki_client.pk_algorithm
import restpki_client.pk_certificate
import restpki_client.pki_brazil_certificate_fields
import restpki_client.pki_italy_certificate_fields
import restpki_client.resource_content_or_reference
import restpki_client.rest_base_error
import restpki_client.rest_error
import restpki_client.rest_pki_client
import restpki_client.rest_pki_error
import restpki_client.rest_unreachable_error
import restpki_client.signature_algorithm_and_value
import restpki_client.signature_explorer
import restpki_client.signature_finisher
import restpki_client.signature_policy_catalog
import restpki_client.signature_policy_identifier
import restpki_client.signature_result
import restpki_client.signature_start_result
import restpki_client.signature_starter
import restpki_client.standard_security_contexts
import restpki_client.standard_signature_policies
import restpki_client.validation
import restpki_client.validation_error
import restpki_client.version
import restpki_client.xml_element_signature_starter
import restpki_client.xml_id_resolution_table
import restpki_client.xml_insertion_options
import restpki_client.xml_signature_finisher
import restpki_client.xml_signature_starter

from restpki_client.apis import Apis
from restpki_client.authentication import Authentication
from restpki_client.authentication_result import AuthenticationResult
from restpki_client.blob_reference import BlobReference
from restpki_client.cades_signature import CadesSignature
from restpki_client.cades_signature import CadesTimestamp
from restpki_client.cades_signature import CadesSignerInfo
from restpki_client.cades_signature_finisher import CadesSignatureFinisher
from restpki_client.cades_signature_starter import CadesSignatureStarter
from restpki_client.color import Color
from restpki_client.detached_resource_xml_signature_starter \
    import DetachedResourceXmlSignatureStarter
from restpki_client.digest_algorithm import DigestAlgorithms
from restpki_client.digest_algorithm import DigestAlgorithm
from restpki_client.digest_algorithm import MD5DigestAlgorithm
from restpki_client.digest_algorithm import SHA1DigestAlgorithm
from restpki_client.digest_algorithm import SHA256DigestAlgorithm
from restpki_client.digest_algorithm import SHA384DigestAlgorithm
from restpki_client.digest_algorithm import SHA512DigestAlgorithm
from restpki_client.digest_algorithm_and_value import DigestAlgorithmAndValue
from restpki_client.file_reference import FileReference
from restpki_client.file_result import FileResult
from restpki_client.full_xml_signature_starter import FullXmlSignatureStarter
from restpki_client.name import Name
from restpki_client.namespace_manager import NamespaceManager
from restpki_client.oids import Oids
from restpki_client.online_resource_xml_signature_starter \
    import OnlineResourceXmlSignatureStarter
from restpki_client.pades_horizontal_align import PadesHorizontalAlign
from restpki_client.pades_measurement_units import PadesMeasurementUnits
from restpki_client.pades_page_optimization import PadesPageOptimization
from restpki_client.pades_signature import PadesSignature
from restpki_client.pades_signature_explorer import PadesSignatureExplorer
from restpki_client.pades_signature_finisher import PadesSignatureFinisher
from restpki_client.pades_signature_starter import PadesSignatureStarter
from restpki_client.pades_signer_info import PadesSignerInfo
from restpki_client.pades_size import PadesSize
from restpki_client.pades_visual_positioning_presets \
    import PadesVisualPositioningPresets
from restpki_client.pades_visual_rectangle import PadesVisualRectangle
from restpki_client.page_orientations import PageOrientations
from restpki_client.paper_sizes import PaperSizes
from restpki_client.pdf_container_definition import PdfContainerDefinition
from restpki_client.pdf_helper import PdfHelper
from restpki_client.pdf_mark import PdfMark
from restpki_client.pdf_mark_element import PdfMarkElement
from restpki_client.pdf_mark_element_type import PdfMarkElementType
from restpki_client.pdf_mark_image import PdfMarkImage
from restpki_client.pdf_mark_image_element import PdfMarkImageElement
from restpki_client.pdf_mark_page_options import PdfMarkPageOptions
from restpki_client.pdf_mark_qr_code_element import PdfMarkQRCodeElement
from restpki_client.pdf_mark_text_element import PdfMarkTextElement
from restpki_client.pdf_marker import PdfMarker
from restpki_client.pdf_text_section import PdfTextSection
from restpki_client.pdf_text_style import PdfTextStyle
from restpki_client.pk_algorithm import SignatureAlgorithms
from restpki_client.pk_algorithm import SignatureAlgorithm
from restpki_client.pk_algorithm import PKAlgorithms
from restpki_client.pk_algorithm import PKAlgorithm
from restpki_client.pk_algorithm import RSASignatureAlgorithm
from restpki_client.pk_algorithm import RSAPKAlgorithm
from restpki_client.pk_certificate import PKCertificate
from restpki_client.pki_brazil_certificate_fields \
    import PkiBrazilCertificateFields
from restpki_client.pki_italy_certificate_fields \
    import PkiItalyCertificateFields
from restpki_client.resource_content_or_reference \
    import ResourceContentOrReference
from restpki_client.rest_base_error import RestBaseError
from restpki_client.rest_error import RestError
from restpki_client.rest_pki_client import RestPkiClient
from restpki_client.rest_pki_error import RestPkiError
from restpki_client.rest_unreachable_error import RestUnreachableError
from restpki_client.signature_algorithm_and_value \
    import SignatureAlgorithmAndValue
from restpki_client.signature_explorer import SignatureExplorer
from restpki_client.signature_finisher import SignatureFinisher
from restpki_client.signature_policy_catalog import SignaturePolicyCatalog
from restpki_client.signature_policy_identifier import SignaturePolicyIdentifier
from restpki_client.signature_result import SignatureResult
from restpki_client.signature_start_result import SignatureStartResult
from restpki_client.signature_starter import SignatureStarter
from restpki_client.standard_security_contexts import StandardSecurityContexts
from restpki_client.standard_signature_policies import StandardSignaturePolicies
from restpki_client.validation import ValidationItem
from restpki_client.validation import ValidationResults
from restpki_client.validation_error import ValidationError
from restpki_client.version import __version__
from restpki_client.xml_element_signature_starter \
    import XmlElementSignatureStarter
from restpki_client.xml_id_resolution_table import XmlIdResolutionTable
from restpki_client.xml_insertion_options import XmlInsertionOptions
from restpki_client.xml_signature_finisher import XmlSignatureFinisher
from restpki_client.xml_signature_starter import XmlSignatureStarter

__all__ = []
__all__ += restpki_client.apis.__all__
__all__ += restpki_client.authentication.__all__
__all__ += restpki_client.authentication_result.__all__
__all__ += restpki_client.blob_reference.__all__
__all__ += restpki_client.cades_signature.__all__
__all__ += restpki_client.cades_signature_finisher.__all__
__all__ += restpki_client.cades_signature_starter.__all__
__all__ += restpki_client.color.__all__
__all__ += restpki_client.detached_resource_xml_signature_starter.__all__
__all__ += restpki_client.digest_algorithm.__all__
__all__ += restpki_client.digest_algorithm_and_value.__all__
__all__ += restpki_client.file_reference.__all__
__all__ += restpki_client.file_result.__all__
__all__ += restpki_client.full_xml_signature_starter.__all__
__all__ += restpki_client.name.__all__
__all__ += restpki_client.namespace_manager.__all__
__all__ += restpki_client.oids.__all__
__all__ += restpki_client.online_resource_xml_signature_starter.__all__
__all__ += restpki_client.pades_horizontal_align.__all__
__all__ += restpki_client.pades_measurement_units.__all__
__all__ += restpki_client.pades_page_optimization.__all__
__all__ += restpki_client.pades_signature.__all__
__all__ += restpki_client.pades_signature_explorer.__all__
__all__ += restpki_client.pades_signature_finisher.__all__
__all__ += restpki_client.pades_signature_starter.__all__
__all__ += restpki_client.pades_signer_info.__all__
__all__ += restpki_client.pades_size.__all__
__all__ += restpki_client.pades_visual_positioning_presets.__all__
__all__ += restpki_client.pades_visual_rectangle.__all__
__all__ += restpki_client.page_orientations.__all__
__all__ += restpki_client.paper_sizes.__all__
__all__ += restpki_client.pdf_container_definition.__all__
__all__ += restpki_client.pdf_helper.__all__
__all__ += restpki_client.pdf_mark.__all__
__all__ += restpki_client.pdf_mark_element.__all__
__all__ += restpki_client.pdf_mark_element_type.__all__
__all__ += restpki_client.pdf_mark_image.__all__
__all__ += restpki_client.pdf_mark_image_element.__all__
__all__ += restpki_client.pdf_mark_page_options.__all__
__all__ += restpki_client.pdf_mark_qr_code_element.__all__
__all__ += restpki_client.pdf_mark_text_element.__all__
__all__ += restpki_client.pdf_marker.__all__
__all__ += restpki_client.pdf_text_section.__all__
__all__ += restpki_client.pdf_text_style.__all__
__all__ += restpki_client.pk_algorithm.__all__
__all__ += restpki_client.pk_certificate.__all__
__all__ += restpki_client.pki_brazil_certificate_fields.__all__
__all__ += restpki_client.pki_italy_certificate_fields.__all__
__all__ += restpki_client.resource_content_or_reference.__all__
__all__ += restpki_client.rest_base_error.__all__
__all__ += restpki_client.rest_error.__all__
__all__ += restpki_client.rest_pki_client.__all__
__all__ += restpki_client.rest_pki_error.__all__
__all__ += restpki_client.rest_unreachable_error.__all__
__all__ += restpki_client.signature_algorithm_and_value.__all__
__all__ += restpki_client.signature_explorer.__all__
__all__ += restpki_client.signature_finisher.__all__
__all__ += restpki_client.signature_policy_catalog.__all__
__all__ += restpki_client.signature_policy_identifier.__all__
__all__ += restpki_client.signature_result.__all__
__all__ += restpki_client.signature_start_result.__all__
__all__ += restpki_client.signature_starter.__all__
__all__ += restpki_client.standard_security_contexts.__all__
__all__ += restpki_client.standard_signature_policies.__all__
__all__ += restpki_client.validation.__all__
__all__ += restpki_client.validation_error.__all__
__all__ += restpki_client.version.__all__
__all__ += restpki_client.xml_element_signature_starter.__all__
__all__ += restpki_client.xml_id_resolution_table.__all__
__all__ += restpki_client.xml_insertion_options.__all__
__all__ += restpki_client.xml_signature_finisher.__all__
__all__ += restpki_client.xml_signature_starter.__all__



from .utils import _base64_encode_string
from .signature_finisher import SignatureFinisher
from .signature_result import SignatureResult


class XmlSignatureFinisher(SignatureFinisher):

    def __init__(self, client):
        super(XmlSignatureFinisher, self).__init__(client)

    def finish(self):
        if not self._token:
            raise Exception('The token was not set')

        if self._signature is None:
            response = self._client.post('Api/XmlSignatures/%s/Finalize' %
                                         self._token)
        else:
            request = {
                'signature': _base64_encode_string(self._signature)
            }
            response = self._client.post('Api/XmlSignatures/%s/SignedBytes' %
                                         self._token)

        return SignatureResult(self._client,
                               {'content': response.get('signedXml')},
                               response.get('certificate', None),
                               response.get('callbackArgument', None))

    def _check_compatibility(self):
        pass

    def _get_api_route(self):
        pass


__all__ = ['XmlSignatureFinisher']



import base64
import binascii

from six import BytesIO


def _base64_encode_string(value):
    """

    This method is just a wrapper that handle the incompatibility between the
    standard_b64encode() method on versions 2 and 3 of Python. On Python 2, a
    string is returned, but on Python 3, a bytes-class instance is returned.

    """
    value_base64 = base64.standard_b64encode(value)
    if type(value_base64) is str:
        return value_base64
    elif type(value_base64) is bytes or type(value_base64) is bytearray:
        return value_base64.decode('ascii')
    return None


def _base64_url_safe_encode_string(value):
    """

    This method is just a wrapper that handle the incompatibility between the
    standard_b64encode() method on versions 2 and 3 of Python. On Python 2, a
    string is returned, but on Python 3, a bytes-class instance is returned.

    """
    value_base64 = base64.urlsafe_b64encode(value)
    if type(value_base64) is str:
        return value_base64
    elif type(value_base64) is bytes or type(value_base64) is bytearray:
        return value_base64.decode('ascii')
    return None


def _base64_decode(value):
    if value is None:
        raise Exception('The provided value is not valid')
    try:
        raw = base64.standard_b64decode(value)
    except (TypeError, binascii.Error):
        raise Exception('The provided certificate is not Base64-encoded')
    return raw


def _copy_stream(src, dst, offset=0, from_where=0, buff_size=4096):
    # Position source ont he required position.
    dst.seek(offset, from_where)
    while True:
        buff = src.read(buff_size)
        if not buff:
            break
        dst.write(buff)


def _get_raw_stream(content):
    stream = BytesIO(content)
    stream.seek(0, 0)
    return stream



from .file_result import FileResult
from .pk_certificate import PKCertificate


class SignatureResult(FileResult):

    def __init__(self,
                 client,
                 file_model,
                 certificate,
                 callback_argument=None):

        super(SignatureResult, self).__init__(client, file_model)
        self.__certificate = None
        if certificate is not None:
            self.__certificate = PKCertificate(certificate)
        self.__callback_argument = callback_argument

    @property
    def certificate(self):
        return self.__certificate

    @certificate.setter
    def certificate(self, value):
        self.__certificate = value

    @property
    def callback_argument(self):
        return self.__callback_argument

    @callback_argument.setter
    def callback_argument(self, value):
        self.__callback_argument = value


__all__ = ['SignatureResult']



# pylint: disable=too-few-public-methods
"""

This module contains the definition of the Apis enumeration.

"""
from .enum import Enum


class Apis(Enum):
    """

    This class represents the Apis enumeration. Each value represents a type
    of REST PKI operation. These operations is used to handle incompatibilities
    between REST PKI APIs.

    The available operation are:
     - START_CADES
     - COMPLETE_CADES
     - START_PADES
     - COMPLETE_PADES
     - MULTIPART_UPLOAD
     - ADD_PDF_MARKS

    """
    START_CADES = None
    COMPLETE_CADES = None
    START_PADES = None
    COMPLETE_PADES = None
    MULTIPART_UPLOAD = None
    ADD_PDF_MARKS = None


# The definition of the enumeration values.
#
# Note: This is made outside the class respecting the execution order, which
# makes the definition of the class be executed before the class itself.
Apis.START_CADES = Apis('StartCades')
Apis.COMPLETE_CADES = Apis('CompleteCades')
Apis.START_PADES = Apis('StartPades')
Apis.COMPLETE_PADES = Apis('CompletePades')
Apis.MULTIPART_UPLOAD = Apis('MultipartUpload')
Apis.ADD_PDF_MARKS = Apis('AddPdfMarks')


__all__ = ['Apis']



from .digest_algorithm import DigestAlgorithm
from .pk_certificate import PKCertificate


class SignatureStartResult(object):

    def __init__(self, response):
        certificate = response.get('certificate', None)
        if certificate is not None:
            self.__certificate = PKCertificate(certificate)
        self.__token = response.get('token', None)
        self.__to_sign_data = response.get('toSignData', None)
        self.__to_sign_hash = response.get('toSignHash', None)
        self.__digest_algorithm_oid = response.get('digestAlgorithmOid', None)
        self.__digest_algorithm = None

    # region "certificate" accessors

    @property
    def certificate(self):
        return self.__get_certificate()

    def __get_certificate(self):
        return self.__certificate

    @certificate.setter
    def certificate(self, value):
        self.__set_certificate(value)

    def __set_certificate(self, value):
        if value is None:
            raise Exception('The provided "certificate" is not valid')
        self.__certificate = value

    # endregion

    # region "token" accessors

    @property
    def token(self):
        return self.__get_token()

    def __get_token(self):
        return self.__token

    @token.setter
    def token(self, value):
        self.__set_token(value)

    def __set_token(self, value):
        if value is None:
            raise Exception('The provided "token" is not valid')
        self.__token = value

    # endregion

    # region "to_sign_data" accessors

    @property
    def to_sign_data(self):
        return self.__get_to_sign_data()

    def __get_to_sign_data(self):
        return self.__to_sign_data

    @to_sign_data.setter
    def to_sign_data(self, value):
        self.__set_to_sign_data(value)

    def __set_to_sign_data(self, value):
        if value is None:
            raise Exception('The provided "to_sign_data" is not valid')
        self.__to_sign_data = value

    # endregion

    # region "to_sign_hash" accessors

    @property
    def to_sign_hash(self):
        return self.__get_to_sign_hash()

    def __get_to_sign_hash(self):
        return self.__to_sign_hash

    @to_sign_hash.setter
    def to_sign_hash(self, value):
        self.__set_to_sign_hash(value)

    def __set_to_sign_hash(self, value):
        if value is None:
            raise Exception('The provided "to_sign_hash" is not valid')
        self.__to_sign_hash = value

    # endregion

    # region "digest_algorithm_oid" accessors

    @property
    def digest_algorithm_oid(self):
        return self.__get_digest_algorithm_oid()

    def __get_digest_algorithm_oid(self):
        return self.__digest_algorithm_oid

    @digest_algorithm_oid.setter
    def digest_algorithm_oid(self, value):
        self.__set_digest_algorithm_oid(value)

    def __set_digest_algorithm_oid(self, value):
        if value is None:
            raise Exception(
                'The provided "digest_algorithm_oid" is not valid')
        self.__digest_algorithm_oid = value

    # endregion

    @property
    def digest_algorithm(self):
        if self.__digest_algorithm is None:
            if self.__digest_algorithm_oid is None:
                return None
            self.__digest_algorithm = DigestAlgorithm\
                .get_instance_by_oid(self.__digest_algorithm_oid)
        return self.__digest_algorithm


__all__ = ['SignatureStartResult']



from .cades_signature import CadesSignerInfo


class PadesSignerInfo(CadesSignerInfo):

    def __init__(self, model):
        super(PadesSignerInfo, self).__init__(model)
        self.__is_document_timestamp = model.get('isDocumentTimestamp', None)
        self.__signature_field_name = model.get('signatureFieldName', None)

    @property
    def is_document_timestamp(self):
        return self.__is_document_timestamp

    @is_document_timestamp.setter
    def is_document_timestamp(self, value):
        self.__is_document_timestamp = value

    @property
    def signature_file_name(self):
        return self.__signature_field_name

    @signature_file_name.setter
    def signature_file_name(self, value):
        self.__signature_field_name = value


__all__ = ['PadesSignerInfo']



from .rest_base_error import RestBaseError


class RestUnreachableError(RestBaseError):

    def __init__(self, verb, url):
        RestBaseError.__init__(self,
                               __name__,
                               "REST action %s %s unreachable" % (verb, url),
                               verb,
                               url)


__all__ = ['RestUnreachableError']



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="restpki-client",
    version='1.1.0',
    author="Ismael Medeiros",
    author_email="IsmaelM@lacunasoftware.com",
    description="Client package for REST PKI",
    long_description=long_description,
    keywords='python pki rest certificate digital signature x509',
    url="https://github.com/LacunaSoftware/RestPkiPythonClient",
    packages=setuptools.find_packages(),
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: Financial and Insurance Industry',
        'Intended Audience :: Information Technology',
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        "Operating System :: OS Independent",
        'Topic :: Security',
        'Topic :: Security :: Cryptography',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    install_requires=[
        'simplejson==3.8.1',
        'requests==2.19.1',
        'six==1.11.0'
    ],
)



class PadesVisualRectangle(object):

    def __init__(self):
        self.__left = None
        self.__top = None
        self.__right = None
        self.__bottom = None
        self.__width = None
        self.__height = None

    @property
    def left(self):
        return self.__left

    @left.setter
    def left(self, value):
        self.__left = value

    @property
    def top(self):
        return self.__top

    @top.setter
    def top(self, value):
        self.__top = value

    @property
    def right(self):
        return self.__right

    @right.setter
    def right(self, value):
        self.__right = value

    @property
    def bottom(self):
        return self.__bottom

    @bottom.setter
    def bottom(self, value):
        self.__bottom = value

    @property
    def width(self):
        return self.__width

    @width.setter
    def width(self, value):
        self.__width = value

    @property
    def height(self):
        return self.__height

    @height.setter
    def height(self, value):
        self.__height = value

    def to_model(self):
        return {
            'left': self.__left,
            'top': self.__top,
            'right': self.__right,
            'bottom': self.__bottom,
            'width': self.__width,
            'height': self.__height
        }


__all__ = ['PadesVisualRectangle']



from .utils import _base64_encode_string


class ResourceContentOrReference(object):

    def __init__(self):
        self.__url = None
        self.__mime_type = None
        self.__content = None

    @property
    def url(self):
        return self.__url

    @url.setter
    def url(self, value):
        self.__url = value

    @property
    def mime_type(self):
        return self.__mime_type

    @mime_type.setter
    def mime_type(self, value):
        self.__mime_type = value

    @property
    def content(self):
        return self.__content

    @content.setter
    def content(self, value):
        self.__content = value

    def to_model(self):
        return {
            'content': _base64_encode_string(self.__content)
            if self.__content is not None else None,
            'url': self.__url,
            'mimeType': self.__mime_type
        }


__all__ = ['ResourceContentOrReference']



class StandardSecurityContexts(object):
    PKI_BRAZIL = '201856ce-273c-4058-a872-8937bd547d36'
    PKI_ITALY = 'c438b17e-4862-446b-86ad-6f85734f0bfe'
    WINDOWS_SERVER = '3881384c-a54d-45c5-bbe9-976b674f5ec7'

    """
    
    Lacuna Test PKI (use for development purposes only!) If you are using an
    on-premises instance of REST PKI (instead of https://pki.rest/), this
    security context might have to be created first.
    
    """
    LACUNA_TEST = '803517ad-3bbc-4169-b085-60053a8f6dbf'

    def __init__(self):
        pass


__all__ = ['StandardSecurityContexts']



import hashlib

from abc import ABCMeta
from abc import abstractmethod
from six import BytesIO

from .enum import Enum
from .oids import Oids


class DigestAlgorithms(Enum):
    MD5 = None
    SHA1 = None
    SHA256 = None
    SHA384 = None
    SHA512 = None
    SHA3_256 = None


DigestAlgorithms.MD5 = DigestAlgorithms('MD5')
DigestAlgorithms.SHA1 = DigestAlgorithms('SHA1')
DigestAlgorithms.SHA256 = DigestAlgorithms('SHA256')
DigestAlgorithms.SHA384 = DigestAlgorithms('SHA384')
DigestAlgorithms.SHA512 = DigestAlgorithms('SHA512')
DigestAlgorithms.SHA3_256 = DigestAlgorithms('SHA3_256')


class DigestAlgorithm(object):
    __metaclass__ = ABCMeta

    MD5 = None
    SHA1 = None
    SHA256 = None
    SHA384 = None
    SHA512 = None
    SHA3_256 = None

    def __init__(self, name, oid, xml_uri, byte_length, api_model):
        self.__name = name
        self.__oid = oid
        self.__xml_uri = xml_uri
        self.__byte_length = byte_length
        self.__api_model = api_model

    def __eq__(self, instance):
        if instance is None:
            return False

        return self.__oid == instance.oid

    @staticmethod
    def _algorithms():
        return [
            DigestAlgorithm.MD5,
            DigestAlgorithm.SHA1,
            DigestAlgorithm.SHA256,
            DigestAlgorithm.SHA384,
            DigestAlgorithm.SHA512,
            DigestAlgorithm.SHA3_256
        ]

    @staticmethod
    def _get_safe_algorithms():
        return [
            DigestAlgorithm.SHA256,
            DigestAlgorithm.SHA384,
            DigestAlgorithm.SHA512,
            DigestAlgorithm.SHA3_256
        ]

    @staticmethod
    def get_instance_by_name(name):
        filtered_list = list(filter(lambda a: a.name == name,
                                    DigestAlgorithm._algorithms()))
        alg = filtered_list[0] if len(filtered_list) > 0 else None

        if alg is None:
            raise Exception('Unrecognized digest algorithm name: %s' % name)

        return alg

    @staticmethod
    def get_instance_by_oid(oid):
        filtered_list = list(filter(lambda a: a.oid == oid,
                                    DigestAlgorithm._algorithms()))
        alg = filtered_list[0] if len(filtered_list) > 0 else None

        if alg is None:
            raise Exception('Unrecognized digest algorithm oid: %s' % oid)

        return alg

    @staticmethod
    def get_instance_by_xml_uri(xml_uri):
        filtered_list = list(filter(lambda a: a.xml_uri == xml_uri,
                                    DigestAlgorithm._algorithms()))
        alg = filtered_list[0] if len(filtered_list) > 0 else None

        if alg is None:
            raise Exception('Unrecognized digest algorithm XML URI: %s' %
                            xml_uri)
        return alg

    @staticmethod
    def get_instance_by_api_model(algorithm):
        if algorithm == DigestAlgorithms.MD5:
            return DigestAlgorithm.MD5
        elif algorithm == DigestAlgorithms.SHA1:
            return DigestAlgorithm.SHA1
        elif algorithm == DigestAlgorithms.SHA256:
            return DigestAlgorithm.SHA256
        elif algorithm == DigestAlgorithms.SHA384:
            return DigestAlgorithm.SHA384
        elif algorithm == DigestAlgorithms.SHA512:
            return DigestAlgorithm.SHA512
        elif algorithm == DigestAlgorithms.SHA3_256:
            return DigestAlgorithm.SHA3_256
        else:
            raise Exception('Unsupported digest algorithm: %s' % algorithm)

    def compute_hash_from_content(self, content, offset=0, count=-1):
        stream = BytesIO(content)
        stream.seek(offset, 0)
        hash_func = self.get_hash_func()
        hash_func.update(stream.read(count))
        return hash_func.digest()

    def compute_hash_from_file(self, file_desc, offset=0, count=-1):
        file_desc.seek(offset, 0)
        hash_func = self.get_hash_func()
        hash_func.update(file_desc.read(count))
        return hash_func.digest()

    def check_length(self, digest_value):
        if len(digest_value) != self.__byte_length:
            raise Exception('A %s digest should contain %s bytes, but a value '
                            'with %s bytes was given' % (self.__name,
                                                         self.__byte_length,
                                                         len(digest_value)))

    @property
    def api_model(self):
        return self.__api_model

    @property
    def name(self):
        return self.__name

    @property
    def oid(self):
        return self.__oid

    @property
    def byte_length(self):
        return self.__byte_length

    @property
    def xml_uri(self):
        return self.__xml_uri

    @abstractmethod
    def get_hash_func(self):
        pass


class MD5DigestAlgorithm(DigestAlgorithm):

    def __init__(self):
        name = 'MD5'
        oid = Oids.MD5
        byte_length = 16
        api_model = DigestAlgorithms.MD5
        xml_uri = 'http://www.w3.org/2001/04/xmldsig-more#md5'
        super(MD5DigestAlgorithm, self)\
            .__init__(name, oid, xml_uri, byte_length, api_model)

    def get_hash_func(self):
        return hashlib.md5()


class SHA1DigestAlgorithm(DigestAlgorithm):

    def __init__(self):
        name = 'SHA-1'
        oid = Oids.SHA1
        byte_length = 20
        api_model = DigestAlgorithms.SHA1
        xml_uri = 'http://www.w3.org/2000/09/xmldsig#sha1'
        super(SHA1DigestAlgorithm, self)\
            .__init__(name, oid, xml_uri, byte_length, api_model)

    def get_hash_func(self):
        return hashlib.sha1()


class SHA256DigestAlgorithm(DigestAlgorithm):

    def __init__(self):
        name = 'SHA-256'
        oid = Oids.SHA256
        byte_length = 32
        api_model = DigestAlgorithms.SHA256
        xml_uri = 'http://www.w3.org/2001/04/xmlenc#sha256'
        super(SHA256DigestAlgorithm, self)\
            .__init__(name, oid, xml_uri, byte_length, api_model)

    def get_hash_func(self):
        return hashlib.sha256()


class SHA384DigestAlgorithm(DigestAlgorithm):

    def __init__(self):
        name = 'SHA-384'
        oid = Oids.SHA384
        byte_length = 48
        api_model = DigestAlgorithms.SHA384
        xml_uri = 'http://www.w3.org/2001/04/xmldsig-more#sha384'
        super(SHA384DigestAlgorithm, self)\
            .__init__(name, oid, xml_uri, byte_length, api_model)

    def get_hash_func(self):
        return hashlib.sha384()


class SHA512DigestAlgorithm(DigestAlgorithm):

    def __init__(self):
        name = 'SHA-512'
        oid = Oids.SHA512
        byte_length = 64
        api_model = DigestAlgorithms.SHA512
        xml_uri = 'http://www.w3.org/2001/04/xmlenc#sha512'
        super(SHA512DigestAlgorithm, self)\
            .__init__(name, oid, xml_uri, byte_length, api_model)

    def get_hash_func(self):
        return hashlib.sha512()


class SHA3DigestAlgorithm(DigestAlgorithm):

    def __init__(self, bit_length):
        if bit_length == 256:
            name = 'SHA3-256'
            oid = Oids.SHA3_256
            byte_length = bit_length / 8
            api_model = DigestAlgorithms.SHA3_256
            xml_uri = 'http://www.w3.org/2007/05/xmldsig-more#sha3-256'
            # https://tools.ietf.org/html/rfc6931#section-2.1.5
            super(SHA3DigestAlgorithm, self)\
                .__init__(name, oid, xml_uri, byte_length, api_model)
        else:
            raise Exception('Bit length not supported')
        self.__bit_length = bit_length

    def get_hash_func(self):
        return hashlib.sha3_256()


DigestAlgorithm.MD5 = MD5DigestAlgorithm()
DigestAlgorithm.SHA1 = SHA1DigestAlgorithm()
DigestAlgorithm.SHA256 = SHA256DigestAlgorithm()
DigestAlgorithm.SHA384 = SHA384DigestAlgorithm()
DigestAlgorithm.SHA512 = SHA512DigestAlgorithm()
DigestAlgorithm.SHA3_256 = SHA3DigestAlgorithm(256)

__all__ = [
    'DigestAlgorithms',
    'DigestAlgorithm',
    'MD5DigestAlgorithm',
    'SHA1DigestAlgorithm',
    'SHA256DigestAlgorithm',
    'SHA384DigestAlgorithm',
    'SHA512DigestAlgorithm'
]
